package digital.laboratory.platform.sewage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageImportLog;
import digital.laboratory.platform.sewage.service.SewageImportLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;


/**
 * 污水导入日志
 *
 * @author Zhang Xiaolong created at  2022-10-06
 * @describe 污水导入日志 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sewage_import_log" )
@Api(value = "sewage_import_log", tags = "114-污水导入日志管理")
public class SewageImportLogController {

    private final SewageImportLogService sewageImportLogService;

	/**
	 * 通过id查询污水导入日志
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
//	@PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_get')" )
	public R getById(@PathVariable("id" ) String id) {
		return R.ok(sewageImportLogService.getById(id));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param sewageImportLog 污水导入日志
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_get')" )
    public R getSewageImportLogPage(Page page, SewageImportLog sewageImportLog, HttpServletRequest theHttpServletRequest) {
	    Principal principal = theHttpServletRequest.getUserPrincipal();
	    DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

	    IPage<SewageImportLog> logList = sewageImportLogService.page(page, Wrappers.<SewageImportLog>query()
//				.eq("create_by", dlpUser.getId())
			    .orderByDesc("create_time")
		);
        return R.ok(logList);
    }

	/**
	 * 列表查询
	 * @param sewageImportLog 污水导入日志
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询")
	@GetMapping("/list" )
//    @PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_get')" )
	public R getSewageImportLogList(SewageImportLog sewageImportLog, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<SewageImportLog> logList = sewageImportLogService.list(Wrappers.<SewageImportLog>query()
//				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(logList);
	}

    /**
     * 新增污水导入日志
     * @param sewageImportLog 污水导入日志
     * @return R
     */
    @ApiOperation(value = "新增污水导入日志 <<--- 正式版将取消", notes = "新增污水导入日志 <<--- 正式版将取消")
    @SysLog("新增污水导入日志" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_add')" )
    public R postAddObject(@RequestBody SewageImportLog sewageImportLog) {
        return R.ok(sewageImportLogService.save(sewageImportLog));
    }

    /**
     * 修改污水导入日志
     * @param sewageImportLog 污水导入日志
     * @return R
     */
    @ApiOperation(value = "修改污水导入日志 <<--- 正式版将取消", notes = "修改污水导入日志 <<--- 正式版将取消")
    @SysLog("修改污水导入日志" )
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_edit')" )
    public R putUpdateById(@RequestBody SewageImportLog sewageImportLog) {
        return R.ok(sewageImportLogService.updateById(sewageImportLog));
    }

    /**
     * 通过id删除污水导入日志
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除污水导入日志 <<--- 正式版将取消", notes = "通过id删除污水导入日志 <<--- 正式版将取消")
    @SysLog("通过id删除污水导入日志" )
    @DeleteMapping("/{id}" )
//    @PreAuthorize("@pms.hasPermission('entrustment_sewage_import_log_del')" )
    public R deleteById(@PathVariable String id) {
        return R.ok(sewageImportLogService.removeById(id));
    }

}
