package digital.laboratory.platform.sewage.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageJobGAB;
import digital.laboratory.platform.othersys.dto.QueryCondition;
import digital.laboratory.platform.sewage.service.*;
import digital.laboratory.platform.sewage.vo.SewageJobGABVO;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.feign.RemoteOrgService;
import digital.laboratory.platform.sys.feign.RemoteUserService;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;


/**
 * 污水检测任务
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sewage_job_gab")
@Api(value = "sewage_job_gab", tags = "111-污水检测任务管理(公安部下发)")
public class SewageJobGABController {

	private final SewageJobService sewageJobService;
	private final SewageJobItemService sewageJobItemService;
	private final SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
	private final SewageTreatmentPlantService sewageTreatmentPlantService;

	private final SewageJobGABService sewageJobGABService;

	private final RemoteUserService remoteUserService;
	private final RemoteOrgService remoteOrgService;

	/**
	 * 通过id查询污水检测任务
	 *
	 * @param id id
	 *
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('SewageJobGet')")
	public R getById(@PathVariable("id") String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		List<SewageJobGABVO> sewageJobGABVOS = sewageJobGABService.getSewageJobGABVOList(Wrappers.<SewageJobGAB>query()
				.eq("id", id)
		);

		if ((sewageJobGABVOS == null) || (sewageJobGABVOS.size() == 0)) {
			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
		}

		SewageJobGABVO sewageJob = sewageJobGABVOS.get(0);
		if (sewageJob == null) {
			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
		}

		return R.ok(sewageJob);
	}

	/**
	 * 分页查询
	 *
	 * @param page      分页对象
	 * @param sewageJob 污水检测任务
	 *
	 * @return
	 */
	@ApiOperation(value = "分页查询任务", notes = "分页查询任务\n" +
			"参数:\n" +
			"page - 分页信息: current, size。例如: current=1&size=10\n" +
			"keyword - 查询关键字, 仅对分中心员工有效。 keyword 查询逻辑如下：\n" +
			"	如果 keyword 与某个检材编号、样本1编号、样本2编号相符，则返回该检材对象\n" +
			"statuses - 状态列表, 例如: statuses=1,2,3\n" +
			"status - 单个状态, 例如: status=1\n" +
			"name - 任务名称")
	@GetMapping("/page")
	@PreAuthorize("@pms.hasPermission('SewageJobList')")
	public R getSewageJobPage(Page page, String keyword, SewageJob sewageJob, @RequestParam(value = "statuses", required = false)List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		if (StrUtil.isNotBlank(keyword)) {
			keyword = keyword.trim();
		}
		final String theKeyword = keyword;


		//IPage<SewageJobVO> thePage = null;
		if (dlpUser.isStaff()) {
			// 是鉴定中心员工

			if (StrUtil.isNotBlank(theKeyword)) {
				// 有提供 keyword, 模糊查询

				// 先查是不是检材编号
				List<SewageJobIdentificationMaterialVO> theList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
						.eq("sjim.im_no", theKeyword)
						.or()
						.eq("sjim.sample1_no", theKeyword)
						.or()
						.eq("sjim.sample2_no", theKeyword)
				);
				if (theList.size()==1) {
					// 查到数据了, 直接返回; =0或>1都不对
					return R.ok(theList.get(0));
				}
			}

			IPage<SewageJobGABVO> thePage = sewageJobGABService.getSewageJobGABVOPage(page, Wrappers.<SewageJobGAB>query()
							.in(statuses!=null && statuses.size()>0, "status", statuses)
							.eq(sewageJob.getStatus()!=null, "status", sewageJob.getStatus())
							.like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
							.like(StrUtil.isNotBlank(theKeyword), "name", theKeyword)
							.orderByDesc("expiration_date")
							.orderByDesc("job_year")
							.orderByDesc("job_season")
							.orderByAsc("name")
			);
			return R.ok(thePage);
		}
		else {
			String jobIdSql = "SELECT ji.job_id FROM b_sewage_job_item ji LEFT JOIN b_sewage_treatment_plant stp ON ji.plant_id=stp.id ";
			if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
				// 省级
				jobIdSql += "WHERE stp.province_code='" + userOrg.getAreaCode() + "'";
			}
			else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
				// 市级
				jobIdSql += "WHERE stp.city_code='" + userOrg.getAreaCode() + "'";
			}
			else {
				// 区级
				jobIdSql += "WHERE stp.district_code='" + userOrg.getAreaCode() + "'";
			}

			IPage<SewageJobGABVO> thePage = sewageJobGABService.getSewageJobGABVOPage(page, Wrappers.<SewageJobGAB>query()
					.in(statuses!=null && statuses.size()>0, "status", statuses)
					.eq(sewageJob.getStatus()!=null, "status", sewageJob.getStatus())
					.like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
					.like(StrUtil.isNotBlank(theKeyword), "name", theKeyword)
					.inSql("id", jobIdSql)       // 只查包含本用户所在区域的污水厂的任务
					//.orderByDesc("expiration_date")
					//.orderByDesc("job_year")
					//.orderByDesc("job_season")
					//.orderByAsc("name")
					.orderByDesc("create_time")
			);
			return R.ok(thePage);
		}

	}

	/**
	 * 列表查询
	 * 鉴定中心员工全部可看
	 * 其他用户只可看到包含有其下污水处理厂的任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询")
	@GetMapping("/list")
	@PreAuthorize("@pms.hasPermission('SewageJobList')")
	public R getSewageJobList(SewageJob sewageJob, @RequestParam(value = "statuses", required = false)List<Integer> statuses, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		final SysOrg userOrg;    // 定义成 final才可以被后面的 Lambda 表达式使用
		R r = remoteOrgService.getById(dlpUser.getOrgId());
		if (r.getCode() == CommonConstants.SUCCESS) {
			userOrg = (SysOrg) r.getData();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}

		List<SewageJobGABVO> list = null;
		if (dlpUser.isStaff()) {
			// 是鉴定中心员工
			list = sewageJobGABService.getSewageJobGABVOList(Wrappers.<SewageJobGAB>query()
//					.eq("launche_org_id", dlpUser.getOrgId())       // 只查询当前用户的机构下的任务
							.in(statuses!=null && statuses.size()>0, "status", statuses)
							.eq(sewageJob.getStatus()!=null, "status", sewageJob.getStatus())
							.like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
							.orderByDesc("expiration_date")
							.orderByDesc("job_year")
							.orderByDesc("job_season")
							.orderByAsc("name")
			);
		}
		else {
			String jobIdSql = "SELECT ji.job_id FROM b_sewage_job_item ji LEFT JOIN b_sewage_treatment_plant stp ON ji.plant_id=stp.id ";
			if (StrUtil.endWith(userOrg.getAreaCode(), "0000")) {
				// 省级
				jobIdSql += "WHERE stp.province_code='" + userOrg.getAreaCode() + "'";
			}
			else if (StrUtil.endWith(userOrg.getAreaCode(), "00")) {
				// 市级
				jobIdSql += "WHERE stp.city_code='" + userOrg.getAreaCode() + "'";
			}
			else {
				// 区级
				jobIdSql += "WHERE stp.district_code='" + userOrg.getAreaCode() + "'";
			}

			list = sewageJobGABService.getSewageJobGABVOList(Wrappers.<SewageJobGAB>query()
					.eq("status", 1)        // 非鉴定中心用户只能看开到发布后的任务
					.like(StrUtil.isNotBlank(sewageJob.getName()), "name", sewageJob.getName())
					.inSql("id", jobIdSql)       // 只查包含本用户所在区域的污水厂的任务
					.orderByDesc("expiration_date")
					.orderByDesc("job_year")
					.orderByDesc("job_season")
					.orderByAsc("name")
			);

		}
		return R.ok(list);
	}


	/**
	 * 新增污水检测任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return R
	 */
//	@ApiOperation(value = "新增污水检测任务", notes = "新增污水检测任务")
//	@SysLog("新增污水检测任务")
//	@PostMapping
//	@PreAuthorize("@pms.hasPermission('SewageJobCreate')")
//	public R postAddObject(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//		if (StrUtil.isBlank(sewageJob.getName())) {
//			return R.failed(sewageJob, "新的污水检测任务创建失败, 任务名称不能为空");
//		}
//		if (sewageJob.getJobYear() == null) {
//			return R.failed(sewageJob, "新的污水检测任务创建失败, 任务年份不能为空");
//		}
//		if (sewageJob.getJobSeason() == null) {
//			return R.failed(sewageJob, "新的污水检测任务创建失败, 任务季度不能为空");
//		}
////		if ((sewageJob.getJobSeason()<1) || (sewageJob.getJobSeason()>4)) {
////			return R.failed(sewageJob, "新的污水检测任务创建失败, 任务季度的有效值为1~4");
////		}
//
//		sewageJob.setId(IdWorker.get32UUID().toUpperCase());
//		sewageJob.setJobNo(sewageJobService.getNewJobNo(sewageJob.getJobYear(), sewageJob.getJobSeason()));
//		sewageJob.setLauncheOrgId(dlpUser.getOrgId());
//		sewageJob.setStatus(0);    // 初始的状态是刚创建
//
//		if (sewageJobService.save(sewageJob)) {
//			return R.ok(sewageJob, "新的污水检测任务创建成功");
//		}
//		else {
//			return R.failed(sewageJob, "新的污水检测任务创建失败");
//		}
//	}

	/**
	 * 修改污水检测任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return R
	 */
	@ApiOperation(value = "修改污水检测任务 <--- 这个接口会被禁止,", notes = "修改污水检测任务 <--- 这个接口会被禁止,")
	@SysLog("修改污水检测任务")
	@PutMapping
//    @PreAuthorize("@pms.hasPermission('SewageJobEdit')" )
	public R putUpdateById(@RequestBody SewageJobGAB sewageJob, HttpServletRequest theHttpServletRequest) {

		return R.ok(sewageJobGABService.updateById(sewageJob));
	}

	/**
	 * 发布者修改污水检测任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return R
	 */
//	@ApiOperation(value = "发布者修改污水检测任务", notes = "发布者修改污水检测任务<br>" +
//			"保存提交的以下属性:\n" +
//			"job_no \n" +
//			"name 任务名称\n" +
//			"job_year 任务所属年份\n" +
//			"job_season 任务所属季度\n" +
//			"description 任务描述\n" +
//			"launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
//			"source 任务来源\n" +
//			"expiration_date 过期时间\n" +
//			"start_date 开始时间\n" +
//			"comments 备注说明\n" +
//			"launche_user_id 发布用户id (强制设置为当前用户id)\n"
//	)
//	@SysLog("发布者修改污水检测任务")
//	@PutMapping("/launcher_save")
//	@PreAuthorize("@pms.hasPermission('SewageJobEdit')")
//	public R launcherSave(@RequestBody SewageJobGAB sewageJob, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//		SewageJobGAB oldSewageJob = sewageJobGABService.getById(sewageJob.getId());
//		if (oldSewageJob == null) {
//			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
//		}
//
//		if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
//			throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
//		}
//
//		if (StrUtil.isBlank(sewageJob.getJobNo())) {
//			sewageJob.setJobNo(sewageJobService.getNewJobNo(sewageJob.getJobYear(), sewageJob.getJobSeason()));
//		}
//
////		if (oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_CREATED.getStatus()     // 刚创建未发布
////				&& oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_REVOCATION.getStatus()   // 被撤回
////				&& oldSewageJob.getStatus() != SewageJobStatusConstants.JOB_STATUS_REFUSED.getStatus()  // 被拒绝
////		)
////		{
////			throw new RuntimeException(String.format("id为 %s 的污水检测任务已经不能修改", sewageJob.getId()));
////		}
//
//		UpdateWrapper<SewageJobGAB> updateWrapper = new UpdateWrapper<SewageJobGAB>();
//
//		updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等
//
//		updateWrapper.set("job_no", sewageJob.getJobNo());
//		updateWrapper.set("name", sewageJob.getName());
//		updateWrapper.set("job_year", sewageJob.getJobYear());
//		updateWrapper.set("job_season", sewageJob.getJobSeason());
//		updateWrapper.set("description", sewageJob.getDescription());
//		updateWrapper.set("launche_org_id", dlpUser.getOrgId());
//		updateWrapper.set("source", sewageJob.getSource());
//		updateWrapper.set("expiration_date", sewageJob.getExpirationDate());
//		updateWrapper.set("start_date", sewageJob.getStartDate());
//		updateWrapper.set("comments", sewageJob.getComments());
//		updateWrapper.set("launche_user_id", dlpUser.getId());
//
//		if (!sewageJobGABService.update(null, updateWrapper)) {
//			throw new RuntimeException("保存污水检测任务信息失败");
//		}
//
//		return R.ok(sewageJobGABService.getById(sewageJob.getId()), "保存污水检测任务信息成功");
//	}


	/**
	 * 发布者发布污水检测任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return R
	 */
//	@ApiOperation(value = "发布者发布污水检测任务", notes = "发布者发布污水检测任务<br>" +
//			"保存提交的以下属性:\n" +
//			"job_no \n" +
//			"name 任务名称\n" +
//			"job_year 任务所属年份\n" +
//			"job_season 任务所属季度\n" +
//			"description 任务描述\n" +
//			"launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
//			"source 任务来源\n" +
//			"expiration_date 过期时间\n" +
//			"start_date 开始时间\n" +
//			"comments 备注说明\n" +
//			"launche_user_id 发布用户id (强制设置为当前用户id)\n"
//	)
//	@SysLog("发布者发布污水检测任务")
//	@PutMapping("/launche")
//	@PreAuthorize("@pms.hasPermission('SewageJobEdit')")
//	public R launche(@RequestBody SewageJobGAB sewageJob, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//		SewageJobGAB oldSewageJob = sewageJobGABService.getById(sewageJob.getId());
//		if (oldSewageJob == null) {
//			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
//		}
//		if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
//			throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
//		}
//		UpdateWrapper<SewageJobGAB> updateWrapper = new UpdateWrapper<SewageJobGAB>();
//		updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等
//		//updateWrapper.set(sewageJob.getJobNo() != null, "name", sewageJob.getName());
//		//updateWrapper.set(sewageJob.getJobYear() != null, "job_year", sewageJob.getJobYear());
//		//updateWrapper.set(sewageJob.getJobSeason() != null, "job_season", sewageJob.getJobSeason());
//		//updateWrapper.set(sewageJob.getDescription() != null, "description", sewageJob.getDescription());
//		updateWrapper.set("launche_org_id", dlpUser.getOrgId());
//		//updateWrapper.set(sewageJob.getSource() != null, "source", sewageJob.getSource());
//		//updateWrapper.set(sewageJob.getExpirationDate() != null, "expiration_date", sewageJob.getExpirationDate());
//		//updateWrapper.set(sewageJob.getStartDate() != null, "start_date", sewageJob.getStartDate());
//		//updateWrapper.set(sewageJob.getComments() != null, "comments", sewageJob.getComments());
//		updateWrapper.set("launche_user_id", dlpUser.getId());
//
//		updateWrapper.set("status", 1);
//		updateWrapper.set("launche_date", LocalDateTime.now());
//
//		if (!sewageJobGABService.update(null, updateWrapper)) {
//			throw new RuntimeException("发布污水检测任务失败");
//		}
//		//生成检材信息
//		System.out.println("开始生成检材信息");
//		sewageJobGABService.generateSewageMaterial(oldSewageJob);
//		return R.ok(sewageJobGABService.getById(sewageJob.getId()), "发布污水检测任务成功");
//	}

	/**
	 * 发布者取消发布污水检测任务
	 *
	 * @param sewageJob 污水检测任务
	 *
	 * @return R
	 */
//	@ApiOperation(value = "发布者取消发布污水检测任务", notes = "发布者取消发布污水检测任务<br>" +
//			"保存提交的以下属性:\n" +
//			"job_no \n" +
//			"name 任务名称\n" +
//			"job_year 任务所属年份\n" +
//			"job_season 任务所属季度\n" +
//			"description 任务描述\n" +
//			"launche_org_id 发布单位id(强制设置为当前用户所属机构的id)\n" +
//			"source 任务来源\n" +
//			"expiration_date 过期时间\n" +
//			"start_date 开始时间\n" +
//			"comments 备注说明\n" +
//			"launche_user_id 发布用户id (强制设置为当前用户id)\n" +
//			"\n" +
//			"如果一个任务，已经录入了检材，则不能取消发布。"
//	)
//	@SysLog("发布者取消发布污水检测任务")
//	@PutMapping("/unlaunche")
//	@PreAuthorize("@pms.hasPermission('SewageJobEdit')")
//	public R unlaunche(@RequestBody SewageJob sewageJob, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//		SewageJob oldSewageJob = sewageJobService.getById(sewageJob.getId());
//		if (oldSewageJob == null) {
//			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", sewageJob.getId()));
//		}
//
//		if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
//			throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", sewageJob.getId()));
//		}
//
//		// 看一下 任务清单的情况
//		String jobId = sewageJob.getId();
//		List<SewageJobItemVO> jiList = sewageJobItemService.getSewageJobItemVOList(Wrappers.<SewageJobItem>query()
//				.eq(StrUtil.isNotBlank(jobId), "ji.job_id", jobId)
//		);
//
//		List<SewageJobIdentificationMaterialVO> jimList = sewageJobIdentificationMaterialService.getSewageJobIdentificationMaterialVOList(Wrappers.<SewageJobIdentificationMaterial>query()
//				.eq(StrUtil.isNotBlank(jobId), "sjim.job_id", jobId)
//		);
//		if (jimList.size()>0) {
//			throw new RuntimeException("这个任务存在已录入的检材信息, 不能撤回了");
//		}
//
//		UpdateWrapper<SewageJob> updateWrapper = new UpdateWrapper<SewageJob>();
//		updateWrapper.eq("id", sewageJob.getId());        // 查询条件是 id 相等
//
//		updateWrapper.set(sewageJob.getJobNo() != null, "name", sewageJob.getName());
//		updateWrapper.set(sewageJob.getJobYear() != null, "job_year", sewageJob.getJobYear());
//		updateWrapper.set(sewageJob.getJobSeason() != null, "job_season", sewageJob.getJobSeason());
//		updateWrapper.set(sewageJob.getDescription() != null, "description", sewageJob.getDescription());
//		updateWrapper.set("launche_org_id", dlpUser.getOrgId());
//		updateWrapper.set(sewageJob.getSource() != null, "source", sewageJob.getSource());
//		updateWrapper.set(sewageJob.getExpirationDate() != null, "expiration_date", sewageJob.getExpirationDate());
//		updateWrapper.set(sewageJob.getStartDate() != null, "start_date", sewageJob.getStartDate());
//		updateWrapper.set(sewageJob.getComments() != null, "comments", sewageJob.getComments());
//		updateWrapper.set("launche_user_id", dlpUser.getId());
//
//		updateWrapper.set("status", 0);
//		updateWrapper.set("launche_date", null);
//
//		if (!sewageJobService.update(null, updateWrapper)) {
//			throw new RuntimeException("取消发布污水检测任务失败");
//		}
//
//		return R.ok(sewageJobService.getById(sewageJob.getId()), "取消发布污水检测任务成功");
//	}


	/**
	 * 通过id删除污水检测任务
	 *
	 * @param id id
	 *
	 * @return R
	 */
//	@ApiOperation(value = "通过id删除污水检测任务", notes = "通过id删除污水检测任务")
//	@SysLog("通过id删除污水检测任务")
//	@DeleteMapping("/{id}")
//	@PreAuthorize("@pms.hasPermission('SewageJobDelete')")
//	@Transactional(rollbackFor = {RuntimeException.class, Exception.class})
//	public R deleteById(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
//		Principal principal = theHttpServletRequest.getUserPrincipal();
//		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
//
//		SewageJob oldSewageJob = sewageJobService.getById(id);
//		if (oldSewageJob == null) {
//			throw new RuntimeException(String.format("没有找到id为 %s 的污水检测任务", id));
//		}
//
//		if (!StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), oldSewageJob.getLauncheOrgId())) {
//			throw new RuntimeException(String.format("你没有权限修改id为 %s 的污水检测任务的数据", id));
//		}
//
//		if (oldSewageJob.getStatus()>0) {
//			throw new RuntimeException(String.format("id为 %s 的污水检测任务已经发布, 只能删除未发布的任务", id));
//		}
//
//		sewageJobItemService.remove(Wrappers.<SewageJobItem>query().eq("job_id", id));
//
//		return R.ok(sewageJobService.removeById(id));
//	}

	/**
	 * 统计各污水厂的送检情况
	 */
	@ApiOperation(value = "统计各污水厂的送检情况",notes="统计各污水厂的送检情况")
	@GetMapping("/querySendData")
	public  R querySendData(String jobId,String areaCode,String materialStatus,String treeLayer,HttpServletRequest theHttpServletRequest){
		Assert.notNull(jobId,"任务ID不能为空");
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		R ret = remoteOrgService.getById(dlpUser.getOrgId());
		String currentUserAreaCode="";
		if (ret.getCode() == CommonConstants.SUCCESS) {
			currentUserAreaCode = ((SysOrg) ret.getData()).getAreaCode();
		}
		else {
			return R.failed(String.format("没有找到 orgId 为 %s 的机构, 请确认用户所属机构的正确性!", dlpUser.getOrgId()));
		}
		int targetTreeLayer=StringUtils.isNotBlank(treeLayer)?Integer.parseInt(treeLayer):5;
		return  R.ok(sewageJobGABService.querySewageJcSendData(jobId,currentUserAreaCode,materialStatus,targetTreeLayer),"获取数据信息成功");
	}

	/**
	 *
	 * 初始化污水系统
	 */
//	@GetMapping("/initSewageJobData")
//	public R initSewageJobData(){
//		//1、清除污水任务之前，需要先清除污水任务明细，污水检材明细，污水厂数据，必须严格按照顺序，防止脏数据的产生
//		//1、清除检材表  2、清除任务明细表，3、清除污水厂表，4、清除任务表
//		List<SewageJob> sewageJobList=sewageJobService.list();
//		for (SewageJob sewageJob:sewageJobList) {
//			//删除该任务对应的明细任务项
//			sewageJobItemService.remove(Wrappers.<SewageJobItem>query().eq("job_id",sewageJob.getId()));
//			//获取该任务下的检材明细
//			sewageJobIdentificationMaterialService.remove(Wrappers.<SewageJobIdentificationMaterial>query().eq("job_id",sewageJob.getId()));
//			sewageJobService.removeById(sewageJob);
//		}
//		//清除污水厂
//		sewageTreatmentPlantService.remove(Wrappers.<SewageTreatmentPlant>query().ne("id","999999"));
//		return R.ok(1,"操作成功");
//	}

	/**
	 ************************************************************************** 获取禁毒平台上的污水任务数据***************************************
	 */

	/**
	 * 根据id 去平台上获取数据
	 * @return
	 */
	@GetMapping("/sync_drug_platform_data/{id}")
//	@PreAuthorize("@pms.hasPermission('SewageJobGet')")
	public R getDataOne(@PathVariable String id, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}
		SewageJobIdentificationMaterial sewageJobIdentificationMaterial = sewageJobGABService.querySewageDataOne(id, dlpUser);
		if (sewageJobIdentificationMaterial != null) {
			return R.ok(sewageJobIdentificationMaterial, "操作成功");
		} else {
			return R.ok("操作失败");
		}
	}


	/**
	 * 从禁毒大数据平台获取污水数据
	 * @return
	 */
	@GetMapping("/sync_drug_platform_data")
	@PreAuthorize("@pms.hasPermission('SewageJobGet')")
	public R getDataFromDrugPlatform(QueryCondition queryCondition, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}
		sewageJobGABService.getSewageJobDataFromDrugPlatform(queryCondition, dlpUser);

		return R.ok("操作成功");
	}

	/**
	 * 签收
	 * @param ids id
	 * @return R
	 */
	@ApiOperation(value = "签收", notes = "签收")
	@SysLog("签收" )
	@PutMapping("/sample/receive" )
//	@PreAuthorize("@pms.hasPermission('FeatureAnalysisAllPermission')" )
	public R receiveSampleList(@RequestBody List<String> ids,
							   @RequestParam("jobId") String jobId,
							   HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}

		if (!StrUtil.isNotBlank(jobId)) {
			throw new RuntimeException("任务id 不能为空");
		}

		SewageJobGAB byId = sewageJobGABService.getById(jobId);
		if (byId == null) {
			throw new RuntimeException(String.format("id 为 %s 的任务找不到", jobId));
		}

		if (ids == null || ids.size() == 0) {
			throw new RuntimeException("id不能为空，请检查是否勾选样品！");
		}

		List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterials = sewageJobIdentificationMaterialService.listByIds(ids);

		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterials) {
			if (sewageJobIdentificationMaterial.getStatus() != 2 ) {
				throw new RuntimeException(String.format("id 为 %s 的样品不在送检中的状态，不能签收", sewageJobIdentificationMaterial.getId()));
			}
		}

		return R.ok(sewageJobGABService.receiveSampleList(ids, dlpUser),
				"签收成功");
	}

	/**
	 * 拒收
	 * @param ids id
	 * @return R
	 */
	@ApiOperation(value = "拒收样品", notes = "拒收样品")
	@SysLog("拒收样品" )
	@PutMapping("/sample/reject" )
//	@PreAuthorize("@pms.hasPermission('FeatureAnalysisAllPermission')" )
	public R rejectReceiveSampleList(@RequestBody List<String> ids,
							   @RequestParam("jobId") String jobId,
							   @RequestParam("reason") String reason,
							   HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}

		if (!StrUtil.isNotBlank(jobId)) {
			throw new RuntimeException("任务id 不能为空");
		}

		SewageJobGAB byId = sewageJobGABService.getById(jobId);
		if (byId == null) {
			throw new RuntimeException(String.format("id 为 %s 的任务找不到", jobId));
		}

		if (ids == null || ids.size() == 0) {
			throw new RuntimeException("id不能为空，请检查是否勾选样品！");
		}

		List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterials = sewageJobIdentificationMaterialService.listByIds(ids);

		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterials) {
			if (sewageJobIdentificationMaterial.getStatus() != 2 ) {
				throw new RuntimeException(String.format("id 为 %s 的样品不在送检中的状态，不能签收", sewageJobIdentificationMaterial.getId()));
			}
		}

		return R.ok(sewageJobGABService.rejectReceiveSample(ids, reason, dlpUser),
				"拒收成功");
	}

	/**
	 * 退回样品
	 * @param ids id
	 * @return R
	 */
	@ApiOperation(value = "退回样品", notes = "退回样品")
	@SysLog("签收" )
	@PutMapping("/sample/return" )
//	@PreAuthorize("@pms.hasPermission('FeatureAnalysisAllPermission')" )
	public R returnSampleList(@RequestBody List<String> ids,
							  @RequestParam("jobId") String jobId,
							  @RequestParam("reason") String reason,
							   HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}

		if (!StrUtil.isNotBlank(jobId)) {
			throw new RuntimeException("任务id 不能为空");
		}

		SewageJobGAB byId = sewageJobGABService.getById(jobId);
		if (byId == null) {
			throw new RuntimeException(String.format("id 为 %s 的任务找不到", jobId));
		}

		if (ids == null || ids.size() == 0) {
			throw new RuntimeException("id不能为空，请检查是否勾选样品！");
		}

		List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterials = sewageJobIdentificationMaterialService.listByIds(ids);

		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterials) {
			if (sewageJobIdentificationMaterial.getStatus() != 3 ) {
				throw new RuntimeException(String.format("id 为 %s 的样品不在签收中的状态，不能退回", sewageJobIdentificationMaterial.getId()));
			}
		}

		return R.ok(sewageJobGABService.returnSample(ids, reason, dlpUser),
				"退回样品成功");
	}

	/**
	 * 样品作废
	 * @param ids id
	 * @return R
	 */
	@ApiOperation(value = "样品作废", notes = "样品作废")
	@SysLog("拒收样品" )
	@PutMapping("/sample/close" )
//	@PreAuthorize("@pms.hasPermission('FeatureAnalysisAllPermission')" )
	public R closeSampleList(@RequestBody List<String> ids,
									 @RequestParam("jobId") String jobId,
									 @RequestParam("reason") String reason,
									 HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (!dlpUser.isStaff()) {
			throw new RuntimeException(String.format("用户名为 %s 的用户没有权限操作该功能", dlpUser.getName()));
		}

		if (!StrUtil.isNotBlank(jobId)) {
			throw new RuntimeException("任务id 不能为空");
		}

		SewageJobGAB byId = sewageJobGABService.getById(jobId);
		if (byId == null) {
			throw new RuntimeException(String.format("id 为 %s 的任务找不到", jobId));
		}

		if (ids == null || ids.size() == 0) {
			throw new RuntimeException("id不能为空，请检查是否勾选样品！");
		}

		List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterials = sewageJobIdentificationMaterialService.listByIds(ids);

		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : sewageJobIdentificationMaterials) {
			if (sewageJobIdentificationMaterial.getStatus() != 2 ) {
				throw new RuntimeException(String.format("id 为 %s 的样品不在送检中的状态，不能签收", sewageJobIdentificationMaterial.getId()));
			}
		}

		return R.ok(sewageJobGABService.closeSample(ids, reason, dlpUser),
				"样品作废成功");
	}

}
