package digital.laboratory.platform.sewage.mapper;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import digital.laboratory.platform.sewage.dto.StatisticsDiffStatusJobDTO;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.vo.SewageJobVO;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 污水检测任务 Mapper 接口
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 Mapper 类
 */
@Mapper
public interface SewageJobMapper extends BaseMapper<SewageJob> {
    IPage<SewageJobVO> getSewageJobVOPage(@Param("page") IPage<SewageJob> page, @Param(Constants.WRAPPER) QueryWrapper<SewageJob> qw);

    IPage<SewageJob> getAllJob(@Param("page") IPage<SewageJob> page, @Param("name") String name);

    List<SewageJobVO> getSewageJobVOList(@Param(Constants.WRAPPER) QueryWrapper<SewageJob> qw);

    List<SewageTreeVO> getSewageSendData(@Param(Constants.WRAPPER) QueryWrapper<SewageTreeVO> qw);

    String getArea(@Param(value = "fullname") String fullname);

    /**
     * 根据任务发布时间查询在这之前的任务，并根据发布时间降序，然后根据limit 取不同数量的值
     * @param startDate
     * @param limit
     * @return
     */
    List<SewageJobVO> querySewageJobByStartDate(@Param("startDate") String startDate, @Param("limit") int limit);

    /**
     * 根据任务状态 统计不同状态下的任务数量
     * @param created 待发布状态
     * @param publish 已发布状态
     * @param stopReceive 已停止接收样本状态
     * @param finished 已完成状态
     * @return
     */
    StatisticsDiffStatusJobDTO statisticsDiffStatusJob(@Param("created") Integer created,
                                                       @Param("publish") Integer publish,
                                                       @Param("stopReceive") Integer stopReceive,
                                                       @Param("finished") Integer finished);

}
