package digital.laboratory.platform.sewage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageJobGAB;
import digital.laboratory.platform.othersys.dto.QueryCondition;
import digital.laboratory.platform.sewage.vo.SewageJobGABVO;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import digital.laboratory.platform.sewage.vo.TreeVo;

import java.util.List;

/**
 * 污水检测任务服务类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 服务类
 */
public interface SewageJobGABService extends IService<SewageJobGAB> {
	/**
	 * 取新的任务号
	 * 前缀为 WS, 加4位 年份, 加1位季度, 加 2 位序号
	 * 格式为: "WS" + jobYear + jobSeason + "nn"
	 * 例如: WS2022201
	 * @param jobYear
	 * @param jobSeason
	 * @return
	 */
	String getNewJobNo(int jobYear, int jobSeason);

	IPage<SewageJobGABVO> getSewageJobGABVOPage(IPage<SewageJobGAB> page, QueryWrapper<SewageJobGAB> qw);
	List<SewageJobGABVO> getSewageJobGABVOList(QueryWrapper<SewageJobGAB> qw);

	//查询统计各个污水厂的检材送检，受理情况
	List<TreeVo> querySewageJcSendData(String jobId, String areaCode, String materialStatus, Integer treeLayer);
	List<SewageTreeVO> querySewageJcSendData(String areaCode, int lv);
	//生成检材
	Boolean generateSewageMaterial(SewageJobGAB sewageJob);


    SewageJobIdentificationMaterial querySewageDataOne(String id, DLPUser dlpUser);

    boolean getSewageJobDataFromDrugPlatform(QueryCondition queryCondition, DLPUser dlpUser);

    boolean receiveSampleList(List<String> ids, DLPUser dlpUser);

    boolean closeSample(List<String> ids, String reason, DLPUser dlpUser);

    boolean rejectReceiveSample(List<String> ids, String rejectReason, DLPUser dlpUser);


	boolean returnSample(List<String> ids, String returnReason, DLPUser dlpUser);
}
