package digital.laboratory.platform.sewage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.sewage.dto.SewageJobIdentificationMaterialDTO;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.vo.SewageJobIdentificationMaterialVO;

import java.util.List;

/**
 * 污水任务的检材信息服务类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水任务的检材信息 服务类
 */
public interface SewageJobIdentificationMaterialService extends IService<SewageJobIdentificationMaterial> {
	/**
	 * 取新的检材编号
	 * 前缀为 WJ, 加任务号后7位, 加6位 污水厂地区码, 加2位顺序号
	 * 格式为: "WJ" + jobYear + jobSeason + "nn" + AreaCode + "nn"
	 * 例如: WJ2022201 520111 001
	 * @param jobId
	 * @param jobItemId
	 * @return
	 */
	String getNewCode(String jobId, String jobItemId);

	SewageJobIdentificationMaterialVO getVOById(String id);

	IPage<SewageJobIdentificationMaterialVO> getSewageJobIdentificationMaterialVOPage(IPage<SewageJobIdentificationMaterial> page, QueryWrapper<SewageJobIdentificationMaterial> qw);
	List<SewageJobIdentificationMaterialVO> getSewageJobIdentificationMaterialVOList(QueryWrapper<SewageJobIdentificationMaterial> qw);

	IPage<SewageJobIdentificationMaterialVO> getSewageJobGABIdentificationMaterialVOPage(IPage<SewageJobIdentificationMaterial> page, QueryWrapper<SewageJobIdentificationMaterial> qw);

	List<SewageJobIdentificationMaterialVO> getSewageJobGABIdentificationMaterialVOList(QueryWrapper<SewageJobIdentificationMaterial> qw);

	String buildSewageSampleLableContent(String id, String sampleNo) throws Exception;

	List<SewageJobIdentificationMaterial> getSewageJobIdentificationMaterialList(List<String> ids);
	//批量送检污水检材
	List<SewageJobIdentificationMaterial> batchSendMaterial(SewageJobIdentificationMaterialDTO sewageJobIdentificationMaterialDTO);
	//作废污水检材
	List<SewageJobIdentificationMaterial> cancellationMaterial(List<String> materialIdList, String causeOfCancellation);
	//获取污水送检联系人
	String getContactsInfo();
	//获取未签收的检材数量
	int getSewageMaterialNoSignCount(List<Integer> materialStatus);

    SewageJobIdentificationMaterial getByImNo(String imNo);
}
