package digital.laboratory.platform.sewage.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.dto.StatisticsDiffStatusJobDTO;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.vo.SewageJobVO;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import digital.laboratory.platform.sewage.vo.TreeVo;
import digital.laboratory.platform.sys.vo.entrustment.MarkersVO;
import digital.laboratory.platform.sys.entity.entrustment.Sample;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * 污水检测任务服务类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 服务类
 */
public interface SewageJobService extends IService<SewageJob> {
	/**
	 * 取新的任务号
	 * 前缀为 WS, 加4位 年份, 加1位季度, 加 2 位序号
	 * 格式为: "WS" + jobYear + jobSeason + "nn"
	 * 例如: WS2022201
	 * @param jobYear
	 * @param jobSeason
	 * @return
	 */
	String getNewJobNo(int jobYear, int jobSeason);

	IPage<SewageJobVO> getSewageJobVOPage(IPage<SewageJob> page, QueryWrapper<SewageJob> qw);
	List<SewageJobVO> getSewageJobVOList(QueryWrapper<SewageJob> qw);

	//查询统计各个污水厂的检材送检，受理情况
	List<TreeVo> querySewageJcSendData(String jobId, String areaCode, String materialStatus, Integer treeLayer);
	List<SewageTreeVO> querySewageJcSendData(String areaCode, int lv);
	//生成检材
	Boolean generateSewageMaterial(SewageJob sewageJob);

	String getNewSampleNo(String IdentificationMaterialNo, int sampleSerialNumber);

	//撤回任务
	Boolean undoTask(String taskID);

    @Transactional
    boolean LeadIntoJob(List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList, Integer year, Integer month, DLPUser dlpUser);

    String getDistrictCode(String provinceName, String cityName, String districtName);

	String getNumber(String prefix);

	/*
	* 查询这个账户（机构）下面所送检且已受理的所有污水检材的数量*/
	Integer getIMQuantity(String orgId);

	/**
	 * 查询各个状态下的检材数量
	 * @param orgId
	 * @return
	 */
	abstract List<MarkersVO>  getSewageMarkers(String orgId);

    Map<String, List> getDetails(DLPUser dlpUser, Integer status);

	Integer getImQuantityForAccept();

    IPage<SewageJob> getJobAndMaterialPage(Page page, Integer status, String keywords);

	/**
	 * 根据任务状态 统计不同状态下的任务数量
	 * @return
	 */
	R<StatisticsDiffStatusJobDTO> statisticsDiffStatusJob();

	/**
	 * 根据任务发布时间查询在这之前的任务，并根据发布时间降序，然后根据limit 取不同数量的值
	 * @param startDate
	 * @param limit
	 * @return
	 */
	R<List<SewageJobVO>> querySewageJobByStartDate(String startDate, int limit);

	/**
	 * 根据污水检材创建样本
	 * @param sewageJobIdentificationMaterial
	 *
	 * @return
	 */
	List<Sample> createSample(SewageJobIdentificationMaterial sewageJobIdentificationMaterial, String dlpUserId);

	List<Sample> createSample(List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterialList, String dlpUserId);
}
