package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.othersys.dto.QueryCondition;
import digital.laboratory.platform.othersys.vo.DataModelVo;
import digital.laboratory.platform.othersys.vo.SynConnInfoVo;
import digital.laboratory.platform.othersys.vo.environmentData.CollectDataVO;
import digital.laboratory.platform.othersys.vo.environmentData.CollectResultVO;
import digital.laboratory.platform.othersys.vo.environmentData.EnvironmentCollectVO;
import digital.laboratory.platform.othersys.vo.environmentData.PackageVO;
import digital.laboratory.platform.sewage.constant.SewageConstants;
import digital.laboratory.platform.sewage.entity.SewageJobGAB;
import digital.laboratory.platform.sewage.entity.SewageJobItem;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.mapper.SewageJobGABMapper;
import digital.laboratory.platform.sewage.mapper.SewageTreatmentPlantMapper;
import digital.laboratory.platform.sewage.properties.ApiPathProperties;
import digital.laboratory.platform.sewage.service.*;
import digital.laboratory.platform.sewage.utils.HttpClientUtils;
import digital.laboratory.platform.sewage.vo.*;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import digital.laboratory.platform.sewage.vo.TreeVo;
import digital.laboratory.platform.sys.feign.RemoteSampleService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 污水检测任务服务实现类
 *
 * @author Zhang Xiaolong created at  2022-07-26
 * @describe 污水检测任务 服务实现类
 */
@Service
@Slf4j
@Transactional
public class SewageJobGABServiceImpl extends ServiceImpl<SewageJobGABMapper, SewageJobGAB> implements SewageJobGABService {

	@Resource
	private SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
	@Resource
	private SewageJobItemService sewageJobItemService;

	@Resource
	private SewageJobService sewageJobService;
	@Resource
	private SewageTreatmentPlantService sewageTreatmentPlantService;

	@Resource
	private SewageTreatmentPlantMapper sewageTreatmentPlantMapper;

	@Resource
	private SynchronizeDataService synchronizeDataService;

	@Resource
	private ApiPathProperties apiPathProperties;

	@Resource
	private RemoteSampleService remoteSampleService;


	/**
	 * 取新的任务号
	 * 前缀为 WS, 加4位 年份, 加1位季度, 加 2 位序号
	 * 格式为: "WS" + jobYear + jobSeason + "nn"
	 * 例如: WS2022201
	 * @param jobYear
	 * @param jobSeason
	 * @return
	 */
	@Override
	public String getNewJobNo(int jobYear, int jobSeason) {
		String prefix = CommonConstants.CODE_PREFIX_SEWAGE_JOB + String.format("%d%d", jobYear, jobSeason);

		List<SewageJobGAB> list = this.list(Wrappers.<SewageJobGAB>query()
						.likeRight("job_no", prefix)        // LIKE '值%'
						.orderByDesc("job_no")
		);

		int newNo = 1;
		if ((list!=null) && (list.size()>0)) {
			SewageJobGAB sj = list.get(0);
			String strMaxNo = StrUtil.removePrefixIgnoreCase(sj.getJobNo(), prefix);
			try {
				int maxno = Integer.parseUnsignedInt(strMaxNo);
				newNo = maxno + 1;
			} catch (NumberFormatException e) {
				// 如果后缀有非数字, 则无视之, 重头编码
				newNo = 1;
			}
		}

		String newCode = "C" + prefix+ String.format("%02d", newNo);

		return newCode;
	}


	@Override
	public IPage<SewageJobGABVO> getSewageJobGABVOPage(IPage<SewageJobGAB> page, QueryWrapper<SewageJobGAB> qw) {
		IPage<SewageJobGABVO> r =baseMapper.getSewageJobGABVOPage(page, qw);
		return r;
	}

	@Override
	public List<SewageJobGABVO> getSewageJobGABVOList(QueryWrapper<SewageJobGAB> qw) {
		List<SewageJobGABVO> list = baseMapper.getSewageJobGABVOList(qw);
		return list;
	}
	///////////////////////*******////////////////////////////

	@Override
	public Boolean generateSewageMaterial(SewageJobGAB sewageJob) {
		//查询该任务下的污水厂
		List<SewageJobItem> taskItemList=sewageJobItemService.list(Wrappers.<SewageJobItem>query()
				.eq("job_id",sewageJob.getId()));

		return genSewageMaterialList(taskItemList,sewageJob);
	}

	@Override
	public List<TreeVo> querySewageJcSendData(String jobId, String areaCode, String materialStatus, Integer treeLayer) {
		int treeLv=0;
		QueryWrapper<SewageTreeVO> updateWrapper=Wrappers.<SewageTreeVO>query();
		updateWrapper.eq(StrUtil.isNotBlank(jobId),"job_id",jobId);
		updateWrapper.eq(StrUtil.isNotBlank(materialStatus),"jc_status",materialStatus);
		if(StrUtil.isNotBlank(areaCode)&&!areaCode.equals("0")){
			if(areaCode.endsWith("0000")){
				updateWrapper.eq("province_code",areaCode);//省
				treeLv=1;
			}else if(areaCode.endsWith("00"))
			{
				updateWrapper.eq("city_code",areaCode);//市
				treeLv=2;
			}else if(!areaCode.endsWith("00"))
			{
				updateWrapper.eq("district_code",areaCode);//市
				treeLv=3;
			}
		}
		List<SewageTreeVO> sewageTreeVOList=this.baseMapper.getSewageSendData(updateWrapper);

		List<TreeVo> retTreeData=new ArrayList<>();
		retTreeData.add(buildTreeDataByList(sewageTreeVOList,treeLv,treeLayer));
		return retTreeData;
	}

	@Override
	public List<SewageTreeVO> querySewageJcSendData(String areaCode,int lv) {

		QueryWrapper<SewageTreeVO> queryWrapper=Wrappers.<SewageTreeVO>query();
		if(lv==1){
			queryWrapper.eq("province_code",areaCode);
		}else if(lv==2)
		{
			queryWrapper.eq("city_code",areaCode);
		}else if(lv==3) {
			queryWrapper.eq("district_code",areaCode);
		}
		List<SewageTreeVO> sewageTreeVOList=this.baseMapper.getSewageSendData(queryWrapper);
		return sewageTreeVOList;
	}

	/**
	 * ====================================== 一下是处理禁毒大数据平台上的环境样品数据   ===========================================================
	 */

	/**
	 * 获取第三方系统中的配置参数
	 *
	 * @return 返回连接的数据结构对象
	 */

	@Override
	public SewageJobIdentificationMaterial querySewageDataOne(String id, DLPUser dlpUser) {
		// 获取系统配置信息
		SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

		// 获取分中心账号token
		String token= synchronizeDataService.getTokenInfo("SubCenter",sysSynConnInfo);

		String remoteUrl = sysSynConnInfo.getApiHost() + apiPathProperties.getTaskListPath()
				+ "/" + id + "?token=" + token;
		System.out.println("=========== remoteUrl =============  " + remoteUrl);

		JSONObject jsonObject = HttpClientUtils.executeGet(remoteUrl, null);

		if (jsonObject.get("statusCode").equals("SUCCESS")) {
			if (jsonObject.get("result") == null) {
				throw new RuntimeException("数据获取出错！");
			}
			CollectDataVO result = JSONObject.toJavaObject((JSONObject)jsonObject.get("result"), CollectDataVO.class);
			List<CollectDataVO> list = new ArrayList<>();
			list.add(result);
			dealDataVOToHere(list, dlpUser);
			return sewageJobIdentificationMaterialService.getById(id);
		}
		return null;
	}


	/**
	 * 从禁毒大数据平台获取污水任务的数据
	 *
	 * @param queryCondition 查询条件
	 * @param dlpUser
	 */
	@Override
	public boolean getSewageJobDataFromDrugPlatform(QueryCondition queryCondition, DLPUser dlpUser) {
//		long start = System.currentTimeMillis();
		List<CollectDataVO> resultData = integrateData(queryCondition);
//		long end = System.currentTimeMillis();
//		long integrateTime = end - start;

//		start = System.currentTimeMillis();
		dealDataVOToHere(resultData, dlpUser);
//		end = System.currentTimeMillis();
//		System.out.println("合并数据耗时 ===== " + integrateTime);
//		System.out.println("保存数据数据耗时 ===== " + (end - start));
		return false;
	}

	// 合并数据
	public List<CollectDataVO> integrateData(QueryCondition queryCondition) {

		List<CollectDataVO> resultData = new ArrayList<>(); // 保存从平台上得到的所有数据

		// 获取系统配置信息
		SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

		// 获取分中心账号token
		String token= synchronizeDataService.getTokenInfo("SubCenter",sysSynConnInfo);

//		String remoteUrl = generateSewageJobListURLPath(sysSynConnInfo, 0, token, queryCondition);
//		System.out.println("remoteUrl ===========================" + remoteUrl);
		// 该接口是分页接口，所以需要需要调用获取所以数据
		for (int i = 0; ; i++) {
			String remoteUrl = generateSewageJobListURLPath(sysSynConnInfo, i, token, queryCondition);
			log.info("remoteUrl ====== {}", remoteUrl);
			System.out.println("remoteUrl ===========================" + remoteUrl);

			Integer pageCount = dealTransferData(remoteUrl, resultData);

			if (pageCount == null) {
				throw new RuntimeException("获取平台数据出错!");
			}
			// 因为大数据平台的分页接口限制在10页以内
			if (i == 10) {
				break;
			}
			if (i >= pageCount - 1) {
				break;
			}
		}

//		Integer pageCount = dealTransferData(remoteUrl, resultData);
//
//		if (pageCount == null ) {
//			throw new RuntimeException("获取平台数据出错!");
//		}
//		if (pageCount > 1) {
//			for (int i = 1; i < pageCount; i++) {
//				remoteUrl = generateSewageJobListURLPath(sysSynConnInfo, i, token, queryCondition);
//				dealTransferData(remoteUrl, resultData);
//			}
//		}
		return resultData;
	}

	/**
	 * 封装调用获取平台的数据方法，以为是分页的可能存在多页数据，返回总页数
	 * @return
	 */
	public Integer dealTransferData(String remoteUrl, List<CollectDataVO> resultData) {
//		long start = System.currentTimeMillis();
		JSONObject responseJson = HttpClientUtils.executeGet(remoteUrl, null);
		EnvironmentCollectVO environmentCollectVO = JSONObject.toJavaObject(responseJson, EnvironmentCollectVO.class); // 不返回的json 转为需要的java 对象
//		long end = System.currentTimeMillis();
//		System.out.println("耗时  list.addAll ==== " + (end -start));
		if (environmentCollectVO == null) {
			log.error("数据获取失败，返回信息为空！");
			throw new RuntimeException("数据获取失败，返回信息为空！");
		} else if (!environmentCollectVO.getStatusCode().equals("SUCCESS")) {
			log.error("数据获取失败，返回信息中 statusCode 为 ERROR！返回信息如下：============== {}", environmentCollectVO.toString());
			throw new RuntimeException("数据获取失败，返回信息中 statusCode 为 ERROR！");
		}
//		System.out.println("平台返回的数据 ==== ：：："  + responseJson);
		if (environmentCollectVO.getResult() != null) {
			CollectResultVO result = environmentCollectVO.getResult();
			List<CollectDataVO> list = result.getList();

			if (list == null || list.size() == 0) {
				throw new RuntimeException("数据出错了");
			}
			if (list.get(0) == null) {
				throw new RuntimeException("数据出错了");
			}
			resultData.addAll(list);

//			System.out.println("**************************** |||||| 从禁毒大数据平台获取的数据是 ===== " + list);
//			dealDataVOToHere(list);
			return result.getPageCount();
		}
		return null;
	}
	//构建任务实体属性
	private SewageJobGAB genTaskEntity(CollectDataVO collectDataVO) {
		int year = Integer.parseInt(collectDataVO.getYear());
		int  quarter = Integer.parseInt(collectDataVO.getQuarter());
		SewageJobGAB sewageJobGAB = new SewageJobGAB();
		sewageJobGAB.setId(IdWorker.get32UUID().toUpperCase());
		sewageJobGAB.setJobNo(this.getNewJobNo(year, quarter));
		sewageJobGAB.setJobYear(year);
		sewageJobGAB.setJobSeason(quarter);
		sewageJobGAB.setName(collectDataVO.getName()); // 任务名称
		sewageJobGAB.setDescription(collectDataVO.getRemark());
		sewageJobGAB.setLauncheOrgId(collectDataVO.getCreateGroupCode());
		sewageJobGAB.setExpirationDate(LocalDateTime.parse(
				collectDataVO.getSampleEndDate() + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
		sewageJobGAB.setStartDate(LocalDateTime.parse(
				collectDataVO.getSampleStartDate() + " 00:00:00", DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
		sewageJobGAB.setStatus(1); // 默认已发布
		sewageJobGAB.setSource("公安部");
		sewageJobGAB.setComments(collectDataVO.getRemark());
		return sewageJobGAB;
	}
	//构建清单项
	private SewageJobItem genJobItemEntity(CollectDataVO collectDataVO,String jobID,String sewageTreatmentPlantId){
		SewageJobItem jobItem = new SewageJobItem();
		jobItem.setId(IdWorker.get32UUID().toUpperCase());
		jobItem.setJobId(jobID);
		jobItem.setPlantId(sewageTreatmentPlantId);
		return jobItem;
	}
	//构建检材
	private SewageJobIdentificationMaterial genMaterialEntity(CollectDataVO collectDataVO,String jobID,String jobItemId){

		SewageJobIdentificationMaterial sewageJobIdentificationMaterial = new SewageJobIdentificationMaterial();
		sewageJobIdentificationMaterial.setId(collectDataVO.getId());
		sewageJobIdentificationMaterial.setJobId(jobID);
		sewageJobIdentificationMaterial.setJobItemId(jobItemId);
		sewageJobIdentificationMaterial.setImNo(collectDataVO.getSampleCode());
		int sampleType = 0;
		if (collectDataVO.getSampleType().equals("workday")) {
			sampleType = 1;
		} else if (collectDataVO.getSampleCode().equals("restday")) {
			sampleType = 2;
		}
		sewageJobIdentificationMaterial.setSampleName(collectDataVO.getSewagePlantName() + "-" + (sampleType == 1 ? "工作日样本" : "节假日样本"));
		sewageJobIdentificationMaterial.setSampleType(sampleType);

		if (StrUtil.isNotBlank(collectDataVO.getDailyFlow())) {
			sewageJobIdentificationMaterial.setDailyFlow(Double.valueOf(collectDataVO.getDailyFlow()));
		}
		sewageJobIdentificationMaterial.setWaterInletNumber(collectDataVO.getWaterInletNumber());
		sewageJobIdentificationMaterial.setWaterInletCount(Integer.valueOf(collectDataVO.getSewagePlantWaterInletCount()));
		sewageJobIdentificationMaterial.setDomesticSewageProportion(Double.valueOf(collectDataVO.getSewagePlantDomesticSewageProportion()));
		sewageJobIdentificationMaterial.setServicePopulation(Double.valueOf(collectDataVO.getSewagePlantServicePopulation()));
		sewageJobIdentificationMaterial.setCollector(collectDataVO.getCollector());
		sewageJobIdentificationMaterial.setCollectorGroup(collectDataVO.getCollectorGroup());
		sewageJobIdentificationMaterial.setComments("禁毒平台数据");

		if (StrUtil.isNotBlank(collectDataVO.getCollectTime())) {
			sewageJobIdentificationMaterial.setCollectTime(LocalDateTime.parse(collectDataVO.getCollectTime() + " 00:00:00",
					DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
		}

		sewageJobIdentificationMaterial.setDeliverMethod("2");
		transferPlatformStatus(collectDataVO, sewageJobIdentificationMaterial);
		return sewageJobIdentificationMaterial;
	}

	public void transferPlatformStatus(CollectDataVO collectDataVO, SewageJobIdentificationMaterial sewageJobIdentificationMaterial) {
		// 判断状态
		Integer status = null;
		String statusStr = "";
		LocalDateTime time = null; // 最后一次操作的时间

		if (collectDataVO.getPackageList() != null && collectDataVO.getPackageList().size() != 0) {
			for (PackageVO packageVO : collectDataVO.getPackageList()) {
				sewageJobIdentificationMaterial.setExpressNumber(packageVO.getExpressNumber());
				sewageJobIdentificationMaterial.setExpressCompany(packageVO.getExpressCompany());
				sewageJobIdentificationMaterial.setDelivererId(packageVO.getSendGroupCode());
				LocalDateTime createTime = LocalDateTime.parse(packageVO.getCreateTime(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
				if (time != null && time.isBefore(createTime)) {
					statusStr = packageVO.getStatus();
					time = createTime;
				}
				if (time == null) {
					time = LocalDateTime.parse(packageVO.getCreateTime(), DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
					statusStr = packageVO.getStatus();
				}
			}
		} else {
			// 如果packagelist 没有数据则该样品是未填报 或者 已采集
			if (collectDataVO.getStatus().equals("created")) {
				status = 0; // 待填报
			}
			if (collectDataVO.getStatus().equals("collected")) {
				status = 1;
			}
			if (collectDataVO.getStatus().equals("closed")) {
				status = -2;
			}
		}
		if (status == null) {
			status = parsePlatformStatusToSewageStatus(statusStr);
		}
		sewageJobIdentificationMaterial.setStatus(status);
		try {
			if (status == 3) {
				sewageJobIdentificationMaterial.setAcceptTime(time);
			}
		} catch (NullPointerException e) {
			System.out.println("异常数据 ************* " + collectDataVO);
		}
	}

	public void dealDataVOToHere(List<CollectDataVO> list, DLPUser dlpUser) {

		// 创建list 保存封装好的数据
	//		List<SewageJobGAB> sewageJobGABS = new ArrayList<>();
	//		List<SewageJobIdentificationMaterial> sewageJobIdentificationMaterials = new ArrayList<>();
		List<SewageJobIdentificationMaterial> materialList=new ArrayList<>();
		// 找出公安部下的所有污水厂
		List<SewageTreatmentPlantVO> sewageTreatmentPlantGABVOList = sewageTreatmentPlantMapper.getSewageTreatmentPlantGABVOList(Wrappers.<SewageTreatmentPlant>query().orderByDesc("create_time"));
		// 根据编号转map
		Map<String, SewageTreatmentPlantVO> treatmentPlantVOMapByNumber = sewageTreatmentPlantGABVOList.stream()
				.collect(Collectors.toMap(SewageTreatmentPlantVO::getNumber, Function.identity()));
		// 根据名称转map, 这里因为名称会重复所以在加上地点名称
		Map<String, SewageTreatmentPlantVO> treatmentPlantVOMapByName = new HashMap<>();
		sewageTreatmentPlantGABVOList.forEach(item -> {
			String key = item.getCityName() + item.getName();
			treatmentPlantVOMapByName.put(key, item);
		});
//		Map<String, SewageTreatmentPlantVO> treatmentPlantVOMapByName = sewageTreatmentPlantGABVOList.stream()
//				.collect(Collectors.toMap(SewageTreatmentPlantVO::getName, Function.identity()));


		//////////////////////////////////////////////////////
		//按任务名称分组
		Map<String,List<CollectDataVO>> retGroup=list.stream().collect(Collectors.groupingBy(item->item.getYear()+item.getQuarter()+item.getName()));

		retGroup.forEach((k,v)->{

			//先保存任务
			CollectDataVO eg=v.get(0);
			//查询这个任务是否存在，如果不存在则插入任务，如果存在，我们就构建清单项，构建完清单项，就构建检材
			List<SewageJobGAB> taskExistList=this.list(Wrappers.<SewageJobGAB>lambdaQuery()
					.eq(SewageJobGAB::getJobYear,Integer.parseInt(eg.getYear()))
					.eq(SewageJobGAB::getJobSeason,Integer.parseInt(eg.getQuarter()))
					.eq(SewageJobGAB::getName,eg.getName()));
			// 任务清单map
			Map<String, SewageJobItem> sewageJobItemMap = new HashMap<>();

			if(taskExistList.size()>0){
				//已经存在
				//保存清单项
				String jobId=taskExistList.get(0).getId();
				Map<String, SewageJobItem> oldSewageJobItemMap = sewageJobItemService.list(Wrappers.<SewageJobItem>query().eq("job_id", jobId))
						.stream().collect(Collectors.toMap(SewageJobItem::getPlantId, Function.identity()));
				v.forEach(collectDataVO -> {

					validSewageTreatmentPlant(collectDataVO, treatmentPlantVOMapByNumber, treatmentPlantVOMapByName);
					String sewageTreatmentPlantId = treatmentPlantVOMapByNumber.get(collectDataVO.getSewagePlantCode()).getId();

					SewageJobItem sewageJobItem = null;
					if (oldSewageJobItemMap.get(sewageTreatmentPlantId) == null) {
						sewageJobItem = genJobItemEntity(collectDataVO, jobId, sewageTreatmentPlantId);
						sewageJobItemMap.put(sewageTreatmentPlantId, sewageJobItem);
					} else {
						sewageJobItem = oldSewageJobItemMap.get(sewageTreatmentPlantId);
					}

					materialList.add(genMaterialEntity(collectDataVO,jobId,sewageJobItem.getId()));
				});
				//保存检材
			}else {
//                sewageJobItemMap = new HashMap<>();
                //不存在
				SewageJobGAB sewageJobGAB=genTaskEntity(eg);
				this.save(sewageJobGAB);
				//保存清单项
				//保存检材
				String jobId=sewageJobGAB.getId();
//				String sewageTreatmentPlantId="";
				v.forEach(collectDataVO -> {
					// 校验污水厂信息，并更新失效的信息
					validSewageTreatmentPlant(collectDataVO, treatmentPlantVOMapByNumber, treatmentPlantVOMapByName);
					String sewageTreatmentPlantId = treatmentPlantVOMapByNumber.get(collectDataVO.getSewagePlantCode()).getId();

					if (sewageJobItemMap.get(sewageTreatmentPlantId) == null) {

						SewageJobItem sewageJobItem = genJobItemEntity(collectDataVO,jobId,sewageTreatmentPlantId);
						sewageJobItemMap.put(sewageTreatmentPlantId, sewageJobItem); // 因为一个任务中一个污水厂只有一条任务清单
					}

					materialList.add(genMaterialEntity(
							collectDataVO,
							jobId,
							sewageJobItemMap.get(sewageTreatmentPlantId).getId()));
				});
			}
			//分贝保存数组即可
			sewageJobItemService.saveBatch(sewageJobItemMap.values());
//			sewageJobIdentificationMaterialService.saveBatch(materialList);
		});
		// 同步从大数据平台中获取到的数据同步到我们系统中的流转系统
		List<SewageJobIdentificationMaterial> collectedMaterialList = materialList.stream().filter(o -> o.getStatus() == 3).collect(Collectors.toList());
		R r = remoteSampleService.enterSampleList(sewageJobService.createSample(collectedMaterialList, dlpUser.getId()));
		if (r.getCode() == CommonConstants.FAIL) {
			log.error("同步到流转系统失败，错误信息：{}", r.getMsg());
		}
		List<String> ids = new ArrayList<>();

		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : materialList) {
			ids.add(sewageJobIdentificationMaterial.getId());
		}

		// 查找之前有没有同步过之前的数据，有则不需要同步，没有则保存到我们的数据库中
		List<SewageJobIdentificationMaterial> materials = sewageJobIdentificationMaterialService.listByIds(ids);
		if (materials != null && materials.size() != 0) {
			materialList.removeAll(materials); // 重写了 污水检材实体类的equals方法和hashcode 方法，认为id 相同，状态相同的实例为相同一个
		}
		Map<String, SewageJobIdentificationMaterial> materialMap = materials.stream()
				.collect(Collectors.toMap(SewageJobIdentificationMaterial::getId, Function.identity()));

		List<SewageJobIdentificationMaterial> updateList = new ArrayList<>();
		List<SewageJobIdentificationMaterial> saveList = new ArrayList<>();
		// 找出要更新的样品列表
		for (SewageJobIdentificationMaterial sewageJobIdentificationMaterial : materialList) {
			if (materialMap.get(sewageJobIdentificationMaterial.getId()) != null) {
				updateList.add(sewageJobIdentificationMaterial);
			} else {
				saveList.add(sewageJobIdentificationMaterial);
			}
		}
		sewageJobIdentificationMaterialService.updateBatchById(updateList);
		sewageJobIdentificationMaterialService.saveBatch(saveList);
		log.info("操作成功，完成了 {} 条数据的保存", materialList.size());
	}

	/***
	 * 校验污水厂的数据信息
	 * @param collectDataVO
	 * @param treatmentPlantVOMapByNumber
	 * @param treatmentPlantVOMapByName
	 */
	private void validSewageTreatmentPlant(CollectDataVO collectDataVO, Map<String, SewageTreatmentPlantVO> treatmentPlantVOMapByNumber, Map<String, SewageTreatmentPlantVO> treatmentPlantVOMapByName) {
		String sewagePlantCode = collectDataVO.getSewagePlantCode();
		String sewagePlantName = collectDataVO.getSewagePlantName();
		if (treatmentPlantVOMapByNumber.get(sewagePlantCode) == null) {
			/**
			 * 目前存在的问题是，以前西北五省的污水厂excel文件上的污水厂信息和从公安部禁毒大数据平台上获取到的不一致，
			 * 比如说 1. 西安创业水务有限公司北石桥污水处理厂  在excel文件的污水厂编号是 61010076，从公安部禁毒大数据平台上的污水厂编号是 61010053
			 * 		 2. 第五污水处理厂  在excel文件的污水厂编号是 61010003，从公安部禁毒大数据平台上的污水厂编号是 61010010
			 * 		 3. 光大水务（咸阳）有限公司  在excel文件的污水厂编号是 61040004，从公安部禁毒大数据平台上的污水厂编号是 61040008
			 */
			// 如果根据编号查询不到，则这里先暂时根据名称去查询，还是查询不到的话则报错
			SewageTreatmentPlantVO sewageTreatmentPlantVO = treatmentPlantVOMapByName.get(collectDataVO.getCityName()+sewagePlantName);
			if (sewageTreatmentPlantVO == null) { // 如果根据名称获取也不存在，则认为该数据不住数据库中，新增到数据库
				SewageTreatmentPlantVO sewageTreatmentPlant = getSewageTreatmentPlantGAB(collectDataVO);
				if (sewageTreatmentPlantMapper.insertSewageTreatmentPlantGAB(sewageTreatmentPlant) == 1) {
					treatmentPlantVOMapByNumber.put(sewageTreatmentPlant.getNumber(), sewageTreatmentPlant);
					treatmentPlantVOMapByName.put(sewageTreatmentPlant.getCityName() + sewageTreatmentPlant.getName(), sewageTreatmentPlant);
					log.info("新增名称为 {} ， 编号为 {} 的污水厂!", sewagePlantName, sewagePlantCode);
				} else {
					log.error("新增名称为 {} ， 编号为 {} 的污水厂失败!", sewagePlantName, sewagePlantCode);
					throw new RuntimeException("新增污水厂失败！");
				}
			} else {
				List<SewageTreatmentPlantVO> plantGABVOListByNumber = sewageTreatmentPlantMapper
						.getSewageTreatmentPlantGABVOList(Wrappers.<SewageTreatmentPlant>query()
								.eq("number", sewagePlantCode));
				if (!sewageTreatmentPlantVO
						.getNumber().equals(sewagePlantCode)
					&& CollUtil.isEmpty(plantGABVOListByNumber)) {
					if (!(sewageTreatmentPlantMapper
							.updateSewageTreatmentPlantGABNumber(sewagePlantCode, sewageTreatmentPlantVO.getId()) == 1)) {
						throw new RuntimeException(String.format("%s 编号更新失败！", sewagePlantName));
					} else {
						sewageTreatmentPlantVO.setNumber(sewagePlantCode);
						treatmentPlantVOMapByNumber.put(sewagePlantCode, sewageTreatmentPlantVO);
						treatmentPlantVOMapByName.put(sewageTreatmentPlantVO.getCityName() + sewagePlantName, sewageTreatmentPlantVO);
					}
				} else {
					throw new RuntimeException(String.format("系统污水厂 %s 的编号%s 和公安部的污水厂编号 %s 冲突了！", sewagePlantName, sewageTreatmentPlantVO.getNumber(), sewagePlantCode));
				}
			}
		}
	}

	/**
	 * 新增从大数据平台同步下来的数据中没有的污水厂数据
	 * @param collectDataVO
	 * @return
	 */
	private SewageTreatmentPlantVO getSewageTreatmentPlantGAB(CollectDataVO collectDataVO) {
		SewageTreatmentPlantVO sewageTreatmentPlant = new SewageTreatmentPlantVO();
		sewageTreatmentPlant.setId(IdWorker.get32UUID());
		sewageTreatmentPlant.setName(collectDataVO.getSewagePlantName());
		sewageTreatmentPlant.setAddress(collectDataVO.getSewagePlantAddress());
		sewageTreatmentPlant.setNumber(collectDataVO.getSewagePlantCode());
		// 设置省市县
		sewageTreatmentPlant.setProvinceCode(collectDataVO.getSewagePlantProvinceCode());
		sewageTreatmentPlant.setProvinceName(collectDataVO.getSewagePlantProvinceName());
		sewageTreatmentPlant.setCityCode(collectDataVO.getSewagePlantCityCode());
		sewageTreatmentPlant.setCityName(collectDataVO.getSewagePlantCityName());
		sewageTreatmentPlant.setDistrictCode(collectDataVO.getSewagePlantAreaCode());
		sewageTreatmentPlant.setDistrictName(collectDataVO.getSewagePlantAreaName());

		sewageTreatmentPlant.setDomesticSewageProportion(Double.valueOf(collectDataVO.getSewagePlantDomesticSewageProportion()));
		sewageTreatmentPlant.setDailyFlowReal(Double.valueOf(collectDataVO.getSewagePlantDailyFlow()));
		sewageTreatmentPlant.setDailyFlowDesign(Double.valueOf(collectDataVO.getSewagePlantDailyFlowPlan()));
		sewageTreatmentPlant.setWaterInletCount(Integer.valueOf(collectDataVO.getSewagePlantWaterInletCount()));
		sewageTreatmentPlant.setServicePopulation(Double.valueOf(collectDataVO.getSewagePlantServicePopulation()));
		return sewageTreatmentPlant;
	}

	/**
	 * 把禁毒大数据平台转换为我们系统的状态
	 * @param platformStatus
	 * @return
	 */
	public Integer parsePlatformStatusToSewageStatus(String platformStatus){

		Integer status = null;
		switch (platformStatus){
			case "canceled": // 取消报送的状态对应我们的未送检
            case "returned": // 已作废
                status = 1;
				break;
			case "created": // 已报送
				status = 2;
				break;
			case "rejected": // 拒收
				status = -1;
				break;
			case "finished": // 签收 || 已受理
				status = 3;
				break;
			case "closed": // 已作废
				status = -2;
				break;
        }

		return status;
	}

	// 生成获取污水数据的url
	private String generateSewageJobListURLPath(SynConnInfoVo sysSynConnInfo, int current, String token, QueryCondition queryCondition) {
		return sysSynConnInfo.getApiHost() + apiPathProperties.getTaskListPath()
				+ "?token=" + token
				+ "&pageIndex=" + current
				+ "&pageSize=" + 100
				+ "&yearIs=" + (queryCondition.getYearIs() != null ? String.valueOf(queryCondition.getYearIs()) : "") // 任务年份精确匹配
				+ "&yearGt=" + (queryCondition.getYearGt() != null ? String.valueOf(queryCondition.getYearGt()) : "") // 任务年份大于
				+ "&yearLt=" + (queryCondition.getYearLt() != null ? String.valueOf(queryCondition.getYearLt()) : "") // 任务年份小于
				+ "&quarterIs=" + (queryCondition.getQuarterIs() != null ? String.valueOf(queryCondition.getQuarterIs()) : "") // 任务季度精确匹配
				+ "&quarterGt=" + (queryCondition.getQuarterGt() != null ? String.valueOf(queryCondition.getQuarterGt()) : "") // 任务年份大于
				+ "&quarterLt=" + (queryCondition.getQuarterLt() != null ? String.valueOf(queryCondition.getQuarterLt()) : "") // 任务年份大于
				+ "&sewagePlantCodeLike=" + (queryCondition.getSewagePlantCodeLike() != null ? queryCondition.getSampleCodeLike() : "") // 污水厂编号模糊匹配
				+ "&sampleCodeLike=" + (queryCondition.getSampleCodeLike() != null ? queryCondition.getSampleCodeLike() : "") // 样品编号模糊查询
				+ "&sewagePlantNameLike=" + (queryCondition.getSewagePlantNameLike() != null ? queryCondition.getSewagePlantNameLike() : "") // 污水厂名称模糊查询
				+ "&nameLike=" + (queryCondition.getNameLike() != null ? queryCondition.getNameLike() : "") // 任务名称模糊查询
				+ "&sampleTypeIs=" + (queryCondition.getSampleTypeIs() != null ? queryCondition.getSampleTypeIs() : "") // 样品类型精确匹配
				+ "&statusIs=" + (queryCondition.getStatusIs() != null ? queryCondition.getStatusIs() : ""); // 任务状态精确匹配;
	}



	/**
	 * 签收样品
	 *
	 * @param ids
	 * @param dlpUser
	 * @return
	 */
	@Override
	public boolean receiveSampleList(List<String> ids, DLPUser dlpUser) {

		// 获取系统配置信息
		SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

		// 获取分中心账号token
		String token= synchronizeDataService.getTokenInfo("SubCenter",sysSynConnInfo);

		// 构建API
		String remoteUrl = sysSynConnInfo.getApiHost()
				+ apiPathProperties.getReceivePath()
				+ "?token=" + token;

		// 请求体参数
		JSONObject paramBody = new JSONObject();
		paramBody.put("list", ids);

		DataModelVo dataModelVo = JSONObject.toJavaObject(
				HttpClientUtils.putJson(remoteUrl, paramBody.toString(), null),
				DataModelVo.class);


		System.out.println("receiveSampleList =========================================== dataModelVo = " + dataModelVo);

		if (dataModelVo.getStatusCode().equals("SUCCESS")) {
			for (String id : ids) {
				querySewageDataOne(id, dlpUser);
			}
			return true;
		}
		return false;
	}

	/**
	 * 作废申请
	 *
	 * @return
	 */
	@Override
	public boolean closeSample(List<String> ids, String reason, DLPUser dlpUser) {

		// 更新作废申请数据
		UpdateWrapper<SewageJobIdentificationMaterial> wrapper = new UpdateWrapper<>();
		wrapper.in("id", ids);
		wrapper.set("cause_of_cancellation", reason);
		if (sewageJobIdentificationMaterialService.update(wrapper)) {
			// 获取系统配置信息
			SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

			// 获取分中心账号token
			String token= synchronizeDataService.getTokenInfo("SubCenter",sysSynConnInfo);
			// 构建API
			String remoteUrl = sysSynConnInfo.getApiHost()
					+ apiPathProperties.getClosePath()
					+ "?token=" + token;

			// 请求体参数
			JSONObject paramBody = new JSONObject();
			paramBody.put("list", ids);
			paramBody.put("reason", reason);

			DataModelVo dataModelVo = JSONObject.toJavaObject(
					HttpClientUtils.putJson(
							remoteUrl,
							paramBody.toString(),
							null),
					DataModelVo.class);

			System.out.println("recheckReceive ================================ dataModelVo = " + dataModelVo);

			if (dataModelVo.getStatusCode().equals("SUCCESS")) {
				for (String id : ids) {
					querySewageDataOne(id, dlpUser);
				}
				return true;
			}
//			return false;
		}
		return false;
	}

	@Override
	public boolean rejectReceiveSample(List<String> ids, String rejectReason, DLPUser dlpUser) {

		// 更新拒收申请数据
		UpdateWrapper<SewageJobIdentificationMaterial> wrapper = new UpdateWrapper<>();
		wrapper.in("id", ids);
		wrapper.set("cause_of_cancellation", rejectReason);

		if (sewageJobIdentificationMaterialService.update(wrapper)) {
			// 获取系统配置信息
			SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

			// 获取分中心账号token
			String token= synchronizeDataService.getTokenInfo("SubCenter",sysSynConnInfo);

			// 构建API
			String remoteUrl = sysSynConnInfo.getApiHost()
					+ apiPathProperties.getRejectPath()
					+ "?token=" + token;

			// 请求体参数
			JSONObject paramBody = new JSONObject();
			paramBody.put("list", ids);

			DataModelVo dataModelVo = JSONObject.toJavaObject(
					HttpClientUtils.putJson(
							remoteUrl,
							paramBody.toString(),
							null),
					DataModelVo.class);

			System.out.println("rejectReceiveSample ============================= dataModelVo = " + dataModelVo);

			if (dataModelVo.getStatusCode().equals("SUCCESS")) {
				for (String id : ids) {
					querySewageDataOne(id, dlpUser);
				}
				return true;
			}
		}

		return false;
	}

	@Override
	public boolean returnSample(List<String> ids, String returnReason, DLPUser dlpUser) {
		// 更新拒收申请数据
		UpdateWrapper<SewageJobIdentificationMaterial> wrapper = new UpdateWrapper<>();
		wrapper.in("id", ids);
		wrapper.set("cause_of_cancellation", returnReason);
		if (sewageJobIdentificationMaterialService.update(wrapper)) {
			// 获取系统配置信息
			SynConnInfoVo sysSynConnInfo = synchronizeDataService.getSysSynConnInfo();

			// 获取分中心账号token
			String token = synchronizeDataService.getTokenInfo("SubCenter", sysSynConnInfo);

			// 构建API
			String remoteUrl = sysSynConnInfo.getApiHost()
					+ apiPathProperties.getReturnPath()
					+ "?token=" + token;

			System.out.println("returnSample url =====" + remoteUrl);
			// 请求体参数
			JSONObject paramBody = new JSONObject();
			paramBody.put("list", ids);

			DataModelVo dataModelVo = JSONObject.toJavaObject(
					HttpClientUtils.putJson(
							remoteUrl,
							paramBody.toString(),
							null),
					DataModelVo.class);

			System.out.println("returnSample  ======================   dataModelVo = " + dataModelVo);

			if (dataModelVo.getStatusCode().equals("SUCCESS")) {
				for (String id : ids) {
					querySewageDataOne(id, dlpUser);
				}
				return true;
			}
		}
		return false;
	}

	/**
	 * ====================================== 以上是处理禁毒大数据平台上的环境样品数据   ===========================================================
	 */

	List<TreeVo> buildTreeVo(Map<String, SewageTreeVO> sourceMap, int lvNo){
		List<TreeVo> treeVoList=new ArrayList<TreeVo>();
		sourceMap.forEach((k, v) -> {
			TreeVo treeVo=new TreeVo();
			treeVo.setId(k);
			treeVo.setDataObj(v);
			if(lvNo==1){
				treeVo.setLabel(v.getProvinceName());
			}else if(lvNo==2){
				treeVo.setLabel(v.getCityName());
			}else if(lvNo==3){
				treeVo.setLabel(v.getDistrictName());
			}else if(lvNo==4) {
				if(v.getWaterFactoryName()!=null)
				{
					treeVo.setLabel(v.getWaterFactoryName());
				}
			}else {
				treeVo.setLabel(v.getJcName());
			}
			//表示没有污水厂，没有污水厂，我们就不加这条数据
			if(v.getWaterFactoryName()!=null)
			{
				treeVoList.add(treeVo);
			}else {
				System.out.println("--------------------------污水厂为空，不添加---------------------");
			}
		});
		return treeVoList;
	}

	List<Integer>  getTreeNodeJcCount(List<TreeVo> childTreeNodeList){
		List<Integer> retList=new ArrayList<Integer>();
		int jcAllCount=0;//总数
		int noFillOut=0;//未填报
		int noSendCount=0;//未送检
		int noAcceptCount=0;//未签收
		int cancelledCount=0;//作废
		for (TreeVo treevo:childTreeNodeList) {
			jcAllCount=jcAllCount+treevo.getJcCount();
			noFillOut=noFillOut+treevo.getNoFillOut();
			noSendCount=noSendCount+treevo.getNoSendCount();
			noAcceptCount=noAcceptCount+treevo.getNoAcceptCount();
			cancelledCount=cancelledCount+treevo.getCancelledCount();
		}
		retList.add(jcAllCount);
		retList.add(noFillOut);
		retList.add(noSendCount);
		retList.add(noAcceptCount);
		retList.add(cancelledCount);
		return retList;
	}
	//将list转换成树形结构

	/**
	 *
	 * @param sourceDataList
	 * @param treeLv   判断树的根节点，比如treeLv=0,那么根节点就是国家毒品实验室陕西分中心，比如=1，就是省级根
	 * @param treeLayer 这个参数表示树的层数，比如是到污水厂，还是只到县
	 * @return
	 */
	TreeVo buildTreeDataByList(List<SewageTreeVO> sourceDataList,int treeLv,int treeLayer){
		//int treeLv=0;
		//int treeLayer=0;
		//根据污水厂的省，市，县
		Map<String, SewageTreeVO> provinceCodeMap = new HashMap<String, SewageTreeVO>();
		Map<String, SewageTreeVO> cityCodeMap = new HashMap<String, SewageTreeVO>();
		Map<String, SewageTreeVO> districtCodeMap = new HashMap<String, SewageTreeVO>();
		Map<String, SewageTreeVO> waterFactoryMap = new HashMap<String, SewageTreeVO>();
		Map<String, SewageTreeVO> jcMap = new HashMap<String, SewageTreeVO>();
		//树的各个层的节点List
		List<TreeVo> baseTreeList = new ArrayList<>();
		List<TreeVo> secondTreeList = new ArrayList<>();
		List<TreeVo> thirdTreeList = new ArrayList<>();
		List<TreeVo> fourthTreeList = new ArrayList<>();
		List<TreeVo> fifthTreeList = new ArrayList<>();
		//将取到的数据进行分解，并将分解到树的各层中去
		for (SewageTreeVO sewageTreeObj:sourceDataList) {
			String provinceCode=sewageTreeObj.getProvinceCode();
			String cityCode=sewageTreeObj.getCityCode();
			String districtCode=sewageTreeObj.getDistrictCode();
			//
			provinceCodeMap.put(provinceCode,sewageTreeObj);//查询有多少个省
			cityCodeMap.put(cityCode,sewageTreeObj);//查询有多少个市
			districtCodeMap.put(districtCode,sewageTreeObj);//查询有多少个县

			waterFactoryMap.put(sewageTreeObj.getWaterFactoryId(),sewageTreeObj);//污水厂
			jcMap.put(sewageTreeObj.getJcId(),sewageTreeObj);
		}
		//构建省的子节点，就是市
		baseTreeList=buildTreeVo(provinceCodeMap,1);
		secondTreeList=buildTreeVo(cityCodeMap,2);
		thirdTreeList=buildTreeVo(districtCodeMap,3);
		fourthTreeList=buildTreeVo(waterFactoryMap,4);
		fifthTreeList=buildTreeVo(jcMap,5);
			//遍历四级节点，找出第四层节点的孩子
		long start = System.currentTimeMillis();
		for(TreeVo fourthTree : fourthTreeList) {
			//获取四级节点前4位
			String factoryCode = fourthTree.getId();

			//遍历5级节点，找出污水厂ID相同的， 找到属于这个污水厂的检材
//			List<TreeVo> tempList1 = fifthTreeList.stream()
//					.filter(e->e.getDataObj().getWaterFactoryId().equals(factoryCode)).collect(Collectors.toList());
//
//			List<TreeVo> tempList2 = fifthTreeList.stream()
//					.filter(e-> e.getDataObj().getWaterFactoryId().equals(factoryCode)&&e.getDataObj().getJcStatus()==0).collect(Collectors.toList());//
//			List<TreeVo> tempList3 = fifthTreeList.stream()
//					.filter(e-> e.getDataObj().getWaterFactoryId().equals(factoryCode)&&e.getDataObj().getJcStatus()==1).collect(Collectors.toList());
//			List<TreeVo> tempList4 = fifthTreeList.stream()
//					.filter(e-> e.getDataObj().getWaterFactoryId().equals(factoryCode)&&e.getDataObj().getJcStatus()==2).collect(Collectors.toList());
//			List<TreeVo> tempList5 = fifthTreeList.stream()
//					.filter(e-> e.getDataObj().getWaterFactoryId().equals(factoryCode)&&e.getDataObj().getJcStatus()==-2).collect(Collectors.toList());
//
//			//设置4级节点的孩子和其他属性
//			fourthTree.setJcCount(tempList1.size());
//			fourthTree.setNoFillOut(tempList2.size());//未填报
//			fourthTree.setNoSendCount(tempList3.size());
//			fourthTree.setNoAcceptCount(tempList4.size());
//			fourthTree.setCancelledCount(tempList5.size());

			int jcCount = 0;
			int noFillOut = 0;
			int noSendCount = 0;
			int noAcceptCount = 0;
			int canceledCount = 0;
			List<TreeVo> tempList1 = new ArrayList<>();
			for (TreeVo fifthTree : fifthTreeList) {
				SewageTreeVO dataObj = fifthTree.getDataObj();

				if (dataObj.getWaterFactoryId().equals(factoryCode)) {
					tempList1.add(fifthTree);
					jcCount++;
					switch (dataObj.getJcStatus()) {
						case 0:
							noFillOut++;
							break;
						case 1:
							noSendCount++;
							break;
						case 2:
							noAcceptCount++;
							break;
						case -2:
							canceledCount++;
							break;
					}
				}
			}

			fourthTree.setJcCount(jcCount);
			fourthTree.setNoFillOut(noFillOut);//未填报
			fourthTree.setNoSendCount(noSendCount);
			fourthTree.setNoAcceptCount(noAcceptCount);
			fourthTree.setCancelledCount(canceledCount);

			if(treeLayer>=4) {
				fourthTree.setChildren(tempList1);
			}
		}
		long end = System.currentTimeMillis();
		System.out.println("耗时 ============ " + (end - start));

			//遍历三级节点，找出第3层节点的孩子
		for(TreeVo threeTree : thirdTreeList) {
				//获取3级节点
				String districtName = threeTree.getLabel();
				//遍历4级节点，找出县级编码相同的污水厂
				List<TreeVo> tempList = fourthTreeList.stream()
						.filter(e->e.getDataObj().getDistrictName().equals(districtName)).collect(Collectors.toList());
				List<Integer> retListCount=getTreeNodeJcCount(tempList);
				threeTree.setTreeNodeStatics(retListCount);
				//设置3级节点的孩子
				if(treeLayer>=4) {
					threeTree.setChildren(tempList);
				}
		}
		//遍历二级节点，找出第2层节点的孩子
		for(TreeVo secondTree : secondTreeList) {
			//获取二级节点前4位
//			String secondTree0_4 = secondTree.getId().substring(0, 4);
			String secondTree0_4 = secondTree.getLabel();
			//遍历3级节点，找出前4位与2级节点相同的
			List<TreeVo> secondChildList = thirdTreeList.stream().filter(e->e.getDataObj().getCityName().equals(secondTree0_4)).collect(Collectors.toList());
			List<Integer> secondTreeCount=getTreeNodeJcCount(secondChildList);
			secondTree.setTreeNodeStatics(secondTreeCount);
			//设置二级节点的三级节点
			secondTree.setChildren(secondChildList);
		}
		//遍历根节点,找出根节点的孩子
		for(TreeVo baseTree : baseTreeList) {
			//获取根节点的前两位
			String baseTree0_2 = baseTree.getLabel();
			//遍历2级节点，找出前两位与根节点相同的
			List<TreeVo> tempList = secondTreeList.stream().filter(e->e.getDataObj().getProvinceName().equals(baseTree0_2))
					.collect(Collectors.toList());
			List<Integer> baseTreeCount=getTreeNodeJcCount(tempList);
			baseTree.setTreeNodeStatics(baseTreeCount);
			//设置根节点的二级节点
			baseTree.setChildren(tempList);
		}
		//以上已经构建好了整个树的数据，我们根据需要选择不同的根来返回树，
		// 如果是分中心看，那就是分中心，如果是省看，根就是省，如果是市看
		// 根就是市，以此类推来返回不同的树根
		TreeVo parentTreeNode=new TreeVo();
		//等于0，表示分中心
		if(treeLv==0){
			parentTreeNode.setLabel(SewageConstants.GY_DRUG_INSPECT_CENTER);
			parentTreeNode.setId("99999");
			parentTreeNode.setChildren(baseTreeList);
			parentTreeNode.setTreeNodeStatics(getTreeNodeJcCount(baseTreeList));
		}else if(treeLv==1){
			parentTreeNode=baseTreeList.get(0);
		}else if(treeLv==2){
			parentTreeNode=secondTreeList.get(0);
		}else if(treeLv==3){
			parentTreeNode=thirdTreeList.get(0);
		}

		return parentTreeNode;
	}

	//生成检材的操作----------------
	private Boolean genSewageMaterialList(List<SewageJobItem> taskItemList,SewageJobGAB sewageJob){

		Map<String,SewageJobItem> mapStpItem=new HashMap<>();
		taskItemList.forEach(item->{
			mapStpItem.put(item.getPlantId(),item);
		});

		List<SewageTreatmentPlant> stpObjBeanList=new ArrayList<>();
		List<String> stpsId=new ArrayList<>();

		mapStpItem.forEach((k,v)->{
			stpsId.add(k);
		});
		if(stpsId.size()>=0)
		{
			//获取污水厂
			stpObjBeanList=sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
					.in("id",stpsId));
		}else {
			throw new RuntimeException(String.format("任务项为0，不能发布任务"));
		}
		List<SewageJobIdentificationMaterial> sewageMatialBeanList=new ArrayList<>();
		stpObjBeanList.forEach(item->{
			List<SewageMaterialVo> sewageMaterialNoList=GenMaterialNo(item,sewageJob);//一个污水厂对应的检材总数
			String taskItemId=mapStpItem.get(item.getId()).getId();
			//拿到每个污水厂生成的编号后，将编号转化为检材对象，方便保存
			sewageMaterialNoList.forEach(materialVo->{
				SewageJobIdentificationMaterial eg=new SewageJobIdentificationMaterial();
				eg.setId(IdWorker.get32UUID());
				eg.setImNo(materialVo.getMaterialNo());
				eg.setJobId(sewageJob.getId());
				eg.setJobItemId(taskItemId);
				eg.setSampleName(materialVo.getMaterialName());
				sewageMatialBeanList.add(eg);
			});
		});
		System.out.println("本次共创建"+sewageMatialBeanList.size()+"份检材");
		//持久化到数据库
		return sewageJobIdentificationMaterialService.saveBatch(sewageMatialBeanList);
	}
	/**
	 * 生成检材的编号
	 * @param sewageTreatmentPlant
	 * @param sewageJob
	 * @return
	 */
	public List<SewageMaterialVo> GenMaterialNo(SewageTreatmentPlant sewageTreatmentPlant, SewageJobGAB sewageJob){
		List<SewageMaterialVo> ImNoList=new ArrayList<SewageMaterialVo>();
		//如果进水口=1，那么规则就是年份+月份+污水厂编号+A/B
		if(sewageTreatmentPlant.getWaterInletCount()<=1)
		{
			String No1=sewageJob.getJobYear()+getQuarterNumber(sewageJob.getJobSeason())+sewageTreatmentPlant.getNumber()+"A";
			String No2=sewageJob.getJobYear()+getQuarterNumber(sewageJob.getJobSeason())+sewageTreatmentPlant.getNumber()+"B";
			SewageMaterialVo eg1 = getSewageMaterialVo(sewageTreatmentPlant, No1,"工作日样本");
			SewageMaterialVo eg2 = getSewageMaterialVo(sewageTreatmentPlant, No2,"节假日样本");
			ImNoList.add(eg1);
			ImNoList.add(eg2);

		}else {
			//如果进水口>=1，那么规则就是年份+月份+污水厂编号+序号+A/B
			for(int j=1;j<sewageTreatmentPlant.getWaterInletCount();j++)
			{
				String No1=sewageJob.getJobYear()+getQuarterNumber(sewageJob.getJobSeason())+sewageTreatmentPlant.getNumber()+j+"A";
				String No2=sewageJob.getJobYear()+getQuarterNumber(sewageJob.getJobSeason())+sewageTreatmentPlant.getNumber()+j+"B";
				SewageMaterialVo eg1 = getSewageMaterialVo(sewageTreatmentPlant, No1,"工作日样本");
				SewageMaterialVo eg2 = getSewageMaterialVo(sewageTreatmentPlant, No2,"节假日样本");
				ImNoList.add(eg1);
				ImNoList.add(eg2);
			}
		}
		return ImNoList;
	}

	private static SewageMaterialVo getSewageMaterialVo(SewageTreatmentPlant sewageTreatmentPlant, String No,String type) {
		SewageMaterialVo eg1=new SewageMaterialVo();
		eg1.setMaterialNo(No);
		eg1.setMaterialName(sewageTreatmentPlant.getName()+"-"+type);
		return eg1;
	}
	//获取月份
	public String getQuarterNumber(int no){
		if(no<10)
		{
			return "0"+no;
		}else
		{
			return no+"";
		}
	}


}
