package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sewage.entity.SewageJob;
import digital.laboratory.platform.sewage.entity.SewageJobIdentificationMaterial;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.entity.*;
import digital.laboratory.platform.sewage.service.SewageJobIdentificationMaterialService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.mapper.SewageJobItemMapper;
import digital.laboratory.platform.sewage.service.SewageJobItemService;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sewage.vo.SewageJobItemVO;
import digital.laboratory.platform.sys.entity.SysOrg;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 污水检测任务清单服务实现类
 *
 * @author Zhang Xiaolong created at  2022-09-13
 * @describe 污水检测任务清单 服务实现类
 */
@Service
public class SewageJobItemServiceImpl extends ServiceImpl<SewageJobItemMapper, SewageJobItem> implements SewageJobItemService {
	@Resource
	SewageJobService sewageJobService;
	@Resource
    SewageTreatmentPlantService sewageTreatmentPlantService;
	@Resource
    SewageJobIdentificationMaterialService sewageJobIdentificationMaterialService;
	@Override
	public IPage<SewageJobItemVO> getSewageJobItemVOPage(IPage<SewageJobItem> page, QueryWrapper<SewageJobItem> qw) {
		IPage<SewageJobItemVO> r =baseMapper.getSewageJobItemVOPage(page, qw);
		return r;
	}

	@Override
	public List<SewageJobItemVO> getSewageJobItemVOList(QueryWrapper<SewageJobItem> qw) {
		List<SewageJobItemVO> list = baseMapper.getSewageJobItemVOList(qw);
		return list;
	}

	@Override
	public List<SewageJobIdentificationMaterial> getSewageMaterialListByStpId(String taskId, List<String> stpIdList) {
		List<SewageJobItem> itemList=this.list(Wrappers.<SewageJobItem>query()
				.in(stpIdList.size()>0,"plant_id",stpIdList)
				.eq("job_id",taskId));
		//
		List<String> itemIdList=new ArrayList<>();
		itemList.forEach(item->{
			itemIdList.add(item.getId());
		});
		List<SewageJobIdentificationMaterial> materialList=sewageJobIdentificationMaterialService
				.list(Wrappers.<SewageJobIdentificationMaterial>query()
						.eq("job_id",taskId)
						.in(itemIdList.size()>0,"job_item_id",itemIdList));

		return materialList;
	}

	//根据任务ID获取任务清单项
	@Override
	public List<String> getSewageJobItemListByTaskID(String taskId,List<String> areaCodeList) {
	  SewageJob sewageJob = sewageJobService.getById(taskId);

	  List<SewageJobItem> sewageJobItemList=this.list(Wrappers.<SewageJobItem>query()
			  .eq("job_id",sewageJob.getId()));
	  List<String> stpIds=new ArrayList<>();
	  sewageJobItemList.forEach(item->{
		  stpIds.add(item.getPlantId());
	  });
	  //已选
	  List<SewageTreatmentPlant> stpObjSelectedList=new ArrayList<>();
	  //总的厂数量
	  List<SewageTreatmentPlant> stpObjUnSelectedList=new ArrayList<>();
	  if(stpIds.size()>0){
		  stpObjSelectedList=sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
				  .in(stpIds.size()>0,"id",stpIds));
		  stpObjUnSelectedList=sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
				  .notIn(stpIds.size()>0,"id",stpIds)
				  .in(areaCodeList.size()>0,"district_code",areaCodeList));
	  }else {
		  stpObjUnSelectedList=sewageTreatmentPlantService.list(Wrappers.<SewageTreatmentPlant>query()
				  .in(areaCodeList.size()>0,"district_code",areaCodeList));
	  }
		String dataString1=JSONArray.toJSONString(stpObjSelectedList);
		String dataString2=JSONArray.toJSONString(stpObjUnSelectedList);
		String dataAreaString=sewageJob.getSewageAreaList();
		List<String> retData=new ArrayList<>();
		retData.add(dataAreaString);
		retData.add(dataString1);
		retData.add(dataString2);
		return retData;
	}
	/**
	 * 批量添加清单项
	 * @param taskJobId
	 * @param sewageTreatmentPlantList
	 * @return
	 * @author:xy
	 * @date:23-0701
	 */
	@Override
	public List<SewageJobItem> batchAddJobItemToTask(String taskJobId, SysOrg userOrg, JSONArray sewageTreatmentPlantList, DLPUser dlpUser) {
		List<SewageJobItem> thisTimeItemList = new ArrayList<SewageJobItem>();//存储本次任务清单项
		Map<String, List<String>>  sewageAreaHashMap= new HashMap<String, List<String>>();//区域
		//构造本次提交的清单项
		sewageTreatmentPlantList.forEach(item->{
			JSONObject stpObject=JSONObject.parseObject(item.toString());
			String stpID=stpObject.getString("stpId");
			String provinceCode=stpObject.getString("provinceCode");
			String cityCode=stpObject.getString("cityCode");
			String districtCode=stpObject.getString("districtCode");
			String stpName=stpObject.getString("stpName");
			//判断如果不是分中心员工，则需要判断是否在管辖范围内
			if (!dlpUser.isStaff()) {
				if (!(StrUtil.equals(userOrg.getAreaCode(), provinceCode)
						|| StrUtil.equals(userOrg.getAreaCode(),cityCode)
						|| StrUtil.equals(userOrg.getAreaCode(), districtCode)
				))
				{
					throw new RuntimeException(String.format("[%s]不在你管辖范围内，你没有权限添加到任务清单中", stpName));
				}
			}
			//构建清单项
			SewageJobItem eg = new SewageJobItem();
			eg.setId(IdWorker.get32UUID().toUpperCase());
			eg.setJobId(taskJobId);
			eg.setCreateByOrgLevel(userOrg.getOrgLevel());
			eg.setPlantId(stpID);
			eg.setComments("1");
			thisTimeItemList.add(eg);
			//设置任务的区域数据
			String haskKey=provinceCode+"-"+cityCode+"-"+districtCode;
			List<String> areaCodeValue=new ArrayList<>();
			areaCodeValue.add(provinceCode);
			areaCodeValue.add(cityCode);
			areaCodeValue.add(districtCode);
			sewageAreaHashMap.put(haskKey,areaCodeValue);
		});
		//根据任务ID判断是否有历史清单
		List<SewageJobItem> historyItemList=this.list(Wrappers.<SewageJobItem>query()
				.eq("job_id",taskJobId));
		if(historyItemList.size()>0){
			this.removeBatchByIds(historyItemList);//如果有就删除
		}
		this.saveBatch(thisTimeItemList);//保存清单项
		//添加之后，更新任务的区域列表
		List<List<String>> taskAreaList=new ArrayList<List<String>>();
		sewageAreaHashMap.forEach((k,v)->{
			taskAreaList.add(v);
		});
		//持久化到数据库
		SewageJob sjob=sewageJobService.getById(taskJobId);
		sjob.setSewageAreaList(JSONArray.toJSONString(taskAreaList));
		sewageJobService.updateById(sjob);
		return thisTimeItemList;
	}
}
