package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.lang.tree.TreeNode;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sewage.utils.QRCodeUtils;
import digital.laboratory.platform.sewage.mapper.SewageTreatmentPlantMapper;
import digital.laboratory.platform.sewage.service.SewageJobService;
import digital.laboratory.platform.sewage.service.SewageTreatmentPlantService;
import digital.laboratory.platform.sewage.vo.SewageTreatmentPlantVO;
import digital.laboratory.platform.sewage.entity.SewageTreatmentPlant;
import digital.laboratory.platform.sewage.vo.SewageTreeVO;
import digital.laboratory.platform.sewage.vo.TreeVo;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;

/**
 * 污水处理厂服务实现类
 *
 * @author Zhang Xiaolong created at  2022-03-22
 * @describe 污水处理厂服务实现类
 */
@Service
public class SewageTreatmentPlantServiceImpl extends ServiceImpl<SewageTreatmentPlantMapper, SewageTreatmentPlant> implements SewageTreatmentPlantService {
	@Resource
	private SewageJobService sewageJobService;
	@Override
	public IPage<SewageTreatmentPlantVO> getSewageTreatmentPlantVOPage(IPage<SewageTreatmentPlant> page, QueryWrapper<SewageTreatmentPlant> qw) {
		IPage<SewageTreatmentPlantVO> r =baseMapper.getSewageTreatmentPlantVOPage(page, qw);
		return r;
	}

	@Override
	public List<SewageTreatmentPlantVO> getSewageTreatmentPlantVOList(QueryWrapper<SewageTreatmentPlant> qw) {
		List<SewageTreatmentPlantVO> list = baseMapper.getSewageTreatmentPlantVOList(qw);
		return list;
	}

	@Override
	public List<TreeVo> getThreeAreaTreeData() {
		List<SewageTreatmentPlant> list=this.list();

		return QRCodeUtils.buildThreeGrade(list);
	}

	@Override
	public TreeVo getSewageTreatmentTreeList(String areaCode) {
		//检查这个areaCode是省，还是市，还是县
		int lv=0;
		List<SewageTreatmentPlant> sewageTreatmentPlantList=new ArrayList<>();
		List<SewageTreeVO> treeInitData=new ArrayList<>();
		String parentNodeName="";


		if(areaCode.equals("0"))
		{
			treeInitData=sewageJobService.querySewageJcSendData(areaCode,0);//查询全部
		}else if(areaCode.endsWith("0000"))
		{
			lv=1;//省级
			treeInitData=sewageJobService.querySewageJcSendData(areaCode,1);//查询省的数据
		}else if(areaCode.endsWith("00"))
		{
			lv=2;//市级
			treeInitData=sewageJobService.querySewageJcSendData(areaCode,2);//查询市的数据
		}else if(!areaCode.endsWith("00"))
		{
			lv=3;//县级
			treeInitData=sewageJobService.querySewageJcSendData(areaCode,3);//查询县的数据
		}

		return QRCodeUtils.buildTreeDataByList(treeInitData,lv);
	}
	private Function<SewageTreatmentPlant, TreeNode<String>> getNodeFunction(int lv) {
		return Obj -> {
			// 扩展属性
			Map<String, Object> extra = new HashMap<>();

			TreeNode<String> node = new TreeNode<>();
			node.setId(Obj.getId());
			if(lv==0){
				node.setParentId("root");
				node.setName(Obj.getProvinceName());
				extra.put("label", Obj.getProvinceName());
				extra.put("areaCode", Obj.getProvinceCode());
			}else if(lv==1){
				node.setParentId(Obj.getProvinceCode());
				node.setName(Obj.getCityName());
				extra.put("label", Obj.getCityName());
				extra.put("areaCode", Obj.getCityCode());
			}else if(lv==2){
				node.setParentId(Obj.getCityCode());
				node.setName(Obj.getDistrictName());
				extra.put("label", Obj.getDistrictName());
				extra.put("areaCode", Obj.getDistrictCode());
			}
			node.setExtra(extra);
			return node;
		};
	}

}
