package digital.laboratory.platform.sewage.service.impl;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.othersys.utils.HttpsUtils;
import digital.laboratory.platform.othersys.vo.*;
import digital.laboratory.platform.sewage.service.SynchronizeDataService;
import digital.laboratory.platform.sys.entity.Dictionary;
import digital.laboratory.platform.sys.entity.DrugLite;
import digital.laboratory.platform.sys.feign.RemoteDictionaryService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @ClassName SynchronizeDataServiceImpl
 * @Description
 * @Author xy
 * @Date 2023/6/13 11:02
 * @Version 1.0
 **/
@Slf4j
@Service
public class SynchronizeDataServiceImpl implements SynchronizeDataService {

    @Resource
    private RemoteDictionaryService remoteDictionaryService;//字典接口


    //一些要用到的公共方法--------------------------------------------------------

    /**
     * 获取第三方系统中的配置参数
     *
     * @return 返回连接的数据结构对象
     */
    @Override
    public SynConnInfoVo getSysSynConnInfo() {
        final SynConnInfoVo synConnInfoVo = new SynConnInfoVo();
        String parentID = "920f9dd9cd14281e8b488a34c649ff70";
        List<Dictionary> retDataList = remoteDictionaryService.insideTreeData(parentID);
        List<Tree<String>> treeData = genTreeData(retDataList, parentID);
        if (treeData.size() > 0) {
            treeData.forEach(sourceObj -> {
                if (sourceObj.get("code").equals("tokenFetch"))//如果是token获取
                {
                    SynConnInfoTokenVo synConnInfoTokenVo = buildTokenInfo(sourceObj.getChildren());
                    synConnInfoVo.setProvinceTokenInfo(synConnInfoTokenVo.getProvinceTokenInfo());
                    synConnInfoVo.setSubCenterTokenInfo(synConnInfoTokenVo.getSubCenterTokenInfo());
                    synConnInfoVo.setTokenInfo(synConnInfoTokenVo);
                }
                if (sourceObj.get("code").equals("dataPath"))//如果是token获取
                {
                    SynConnInfoDataPathVo synConnInfoDataPathVo = buildDataPath(sourceObj.getChildren());
                    synConnInfoVo.setApiHost(synConnInfoDataPathVo.getApiHost());
                    synConnInfoVo.setDataPathInfo(synConnInfoDataPathVo);
                }
            });

        } else {
            System.out.println("没有获取到连接信息，请检查参数");
        }
        return synConnInfoVo;
    }

    /**
     * 构造token对象信息
     *
     * @param tokenAbout
     * @return
     */
    public SynConnInfoTokenVo buildTokenInfo(List<Tree<String>> tokenAbout) {
        SynConnInfoTokenVo synConnInfoTokenVo = new SynConnInfoTokenVo();
        tokenAbout.forEach(itemObj -> {
            String retKey = itemObj.get("label").toString();
            String retValue = itemObj.get("code").toString();
            String keyID = itemObj.get("id").toString();
            switch (retKey) {
                case "sx_sub_center_token":
                    synConnInfoTokenVo.setSubCenterTokenInfo(retValue);
                    synConnInfoTokenVo.setSubCenterTokenID(keyID);
                    break;
                case "sx_province_token":
                    synConnInfoTokenVo.setProvinceTokenInfo(retValue);
                    synConnInfoTokenVo.setProvinceTokenID(keyID);
                    break;
                case "accessKey":
                    synConnInfoTokenVo.setAccessSubCenterKey(retValue);
                    break;
                case "accessSecret":
                    synConnInfoTokenVo.setAccessSubCenterSecret(retValue);
                    break;
                case "accessProvinceKey":
                    synConnInfoTokenVo.setAccessProvinceKey(retValue);
                    break;
                case "accessProvinceSecret":
                    synConnInfoTokenVo.setAccessProvinceSecret(retValue);
                    break;
            }
        });
        return synConnInfoTokenVo;
    }
    //

    /**
     * 构建数据路径信息
     *
     * @param dataPathAbout
     * @return
     */
    public SynConnInfoDataPathVo buildDataPath(List<Tree<String>> dataPathAbout) {
        SynConnInfoDataPathVo synConnInfoDataPathVo = new SynConnInfoDataPathVo();
        dataPathAbout.forEach(itemObj -> {
            String retKey = itemObj.get("label").toString();
            String retValue = itemObj.get("code").toString();
            switch (retKey) {
                case "ApiHost":
                    synConnInfoDataPathVo.setApiHost(retValue);
                    break;
                case "tokenPath":
                    synConnInfoDataPathVo.setTokenPath(retValue);
                    break;
                case "entrustDataPath":
                    synConnInfoDataPathVo.setEntrustDataPath(retValue);
                    break;
                case "wtsStatus":
                    synConnInfoDataPathVo.setWtsStatus(retValue);
                    break;
                case "uploadSewageJobDataPath":
                    synConnInfoDataPathVo.setUploadSewageJobDataUrl(retValue);
                    break;
                case "province_audit_url":
                    synConnInfoDataPathVo.setProvinceAuditUrl(retValue);
                    break;
                case "sx_subcenter_audit_url":
                    synConnInfoDataPathVo.setSxSubCenterAuditUrl(retValue);
                    break;
                case "sx_subcenter_accept_url":
                    synConnInfoDataPathVo.setSxSubCenterAcceptUrl(retValue);
                    break;
            }
        });
        return synConnInfoDataPathVo;
    }

    //根据数组返回一个树形结构
    public List<Tree<String>> genTreeData(List<Dictionary> dictionaryList, String parentID) {
        List<TreeNode<String>> collect = dictionaryList
                .stream()
                .map(getNodeFunction()).collect(Collectors.toList());
        return TreeUtil.build(collect, parentID);
    }

    //构造树形节点数据
    private Function<Dictionary, TreeNode<String>> getNodeFunction() {
        return dictionary -> {
            TreeNode<String> node = new TreeNode<>();
            node.setId(dictionary.getId());
            node.setParentId(dictionary.getType());
            node.setName(dictionary.getLabel());
            // 扩展属性
            Map<String, Object> extra = new HashMap<>();
            extra.put("type", dictionary.getType());
            extra.put("sort", dictionary.getSort());
            extra.put("code", dictionary.getCode());
            extra.put("label", dictionary.getLabel());
            extra.put("comments", dictionary.getComments());
            node.setExtra(extra);
            return node;
        };
    }

    /**
     * 根据禁毒系统中的检材序号，查出同步到我们系统中的物证ID
     *
     * @param index
     * @param evidIdList
     * @return
     */
    public String getEvidIdByIndex(String index, List<Map<String, String>> evidIdList) {

        String retEvid = "";
        for (Map<String, String> mp : evidIdList) {
            String evidIndex = mp.entrySet().stream().findFirst().map(Map.Entry::getKey).get();
            if (evidIndex.equals(index)) {
                if (retEvid.length() > 0) {
                    System.out.println("注意：出现重复的物证ID----------------");
                }
                retEvid = mp.entrySet().stream().findFirst().map(Map.Entry::getValue).get();
            }
        }
        return retEvid;
    }

    /**
     * 根据label和type 查到code
     *
     * @param dictLabel
     * @param parentType
     * @return
     */
    public String getDictCode(String dictLabel, String parentType) {
        String source = "";
        R sourceR = remoteDictionaryService.queryDictionaryVO(dictLabel, parentType);
        if (sourceR.getCode() == CommonConstants.SUCCESS) {
            source = (String) sourceR.getData();
        }
        return source;
    }




    /**
     * 获取token 并更新字典
     *
     * @param remoteUrl
     * @param tokenPath
     * @param accessKey
     * @param accessSecret
     * @param tokenDictID
     * @return
     */
    public String getTokenAndUpdate(String remoteUrl, String tokenPath, String accessKey, String accessSecret, String tokenDictID) {
        String retToken = "";
        Dictionary dictToken = remoteDictionaryService.getDictByID(tokenDictID);
        DataModelVo dataModelVoTmp = HttpsUtils.fetchTokenFromRemote(remoteUrl, tokenPath, accessKey, accessSecret);
        if (dataModelVoTmp.getStatusCode().equals("SUCCESS")) {
            retToken = dataModelVoTmp.getResult().getToken();
            System.out.println("重新申请第三方的数据token成功--");
            //将token 存储起来
            dictToken.setCode(retToken);
            remoteDictionaryService.updateById(dictToken);
            System.out.println("更新第三方数据token成功--");
        } else {
            System.out.println("获取第三方token失败，请检查网络或参数");
        }
        return retToken;
    }

    /**
     * 根据参数不同，获取访问单位的token
     *
     * @param whoUse
     * @return
     */
    @Override
    public String getTokenInfo(String whoUse, SynConnInfoVo synConnInfoVo) {
        String retToken = "";
        String accCheckTokenPath = "/api/sample/drug/task";
        //如果是分中心申请
        if (whoUse.equals("SubCenter")) {

            DataModelVo retData = HttpsUtils.checkTokenValid(synConnInfoVo.getApiHost(), accCheckTokenPath, synConnInfoVo.getSubCenterTokenInfo());
            if (retData.getStatusCode().equals("UNAUTHORIZED")) {
                //已经过期
                retToken = getTokenAndUpdate(synConnInfoVo.getApiHost(), synConnInfoVo.getDataPathInfo().getTokenPath(),
                        synConnInfoVo.getTokenInfo().getAccessSubCenterKey().trim(), synConnInfoVo.getTokenInfo().getAccessSubCenterSecret().trim(),
                        synConnInfoVo.getTokenInfo().getSubCenterTokenID());
            } else {
                retToken = synConnInfoVo.getSubCenterTokenInfo();
            }

        } else {
            //否则就认为是省总队申请
            DataModelVo retData = HttpsUtils.checkTokenValid(synConnInfoVo.getApiHost(), accCheckTokenPath, synConnInfoVo.getProvinceTokenInfo());
            if (retData.getStatusCode().equals("UNAUTHORIZED")) {
                //已经过期
                retToken = getTokenAndUpdate(synConnInfoVo.getApiHost(), synConnInfoVo.getDataPathInfo().getTokenPath(),
                        synConnInfoVo.getTokenInfo().getAccessProvinceKey().trim(), synConnInfoVo.getTokenInfo().getAccessProvinceSecret().trim(),
                        synConnInfoVo.getTokenInfo().getProvinceTokenID());
            } else {
                retToken = synConnInfoVo.getProvinceTokenInfo();
            }
        }
        return retToken;
    }

    /**
     * 获取远程系统中的数据
     *
     * @return DataModelVo  远程系统中获取到的数据对象
     */
    public DataModelVo fetchDataFromRemote(String entrustStatus) {
        SynConnInfoVo synConnInfoVo = getSysSynConnInfo();
        String token = getTokenInfo("SubCenter", synConnInfoVo);
        String remoteUrl = synConnInfoVo.getApiHost();
        String wtDataPath = synConnInfoVo.getDataPathInfo().getEntrustDataPath();
        String wtStatus = entrustStatus;//synConnInfoVo.getDataPathInfo().getWtsStatus();
        DataModelVo retDataModelVo = HttpsUtils.fetchWtDataFromRemote(remoteUrl, wtDataPath, token, wtStatus);
        return retDataModelVo;
    }

    //毒品筛查目标物转换函数
    private List<DrugLite> getDrugLiteList(MaterialVo materialVo) {
        List<DrugLite> retDruglist = new ArrayList<>();
        List<String> dta1 = materialVo.getTargetObjectTypeList();
        dta1.forEach(item -> {
            DrugLite eg = new DrugLite();
            eg.setCode(item);
            eg.setAlias(item);
            eg.setName(item);
            retDruglist.add(eg);
        });
        return retDruglist;
    }
}
